/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "ASN1_cert.h"


ASN1_SEQUENCE(NEWPKI_CERT_REQUEST) = {
	ASN1_SIMPLE(NEWPKI_CERT_REQUEST, id, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_CERT_REQUEST, request, X509_REQ),
	ASN1_SIMPLE(NEWPKI_CERT_REQUEST, validity, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_CERT_REQUEST, ldap_uid, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(NEWPKI_CERT_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_CERT_REQUEST)


ASN1_SEQUENCE(NEWPKI_REV_REQUEST) = {
	ASN1_SIMPLE(NEWPKI_REV_REQUEST, id, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_REV_REQUEST, serial, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_REV_REQUEST, reason, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_REV_REQUEST, ldap_uid, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(NEWPKI_REV_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_REV_REQUEST)

ASN1_SEQUENCE(NEWPKI_PUB_REQUEST_BODY_REV) = {
	ASN1_SIMPLE(NEWPKI_PUB_REQUEST_BODY_REV, cert, X509),
	ASN1_SIMPLE(NEWPKI_PUB_REQUEST_BODY_REV, revdate, ASN1_UTCTIME),
}ASN1_SEQUENCE_END(NEWPKI_PUB_REQUEST_BODY_REV)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PUB_REQUEST_BODY_REV)

ASN1_CHOICE(NEWPKI_PUB_REQUEST_BODY) = {
	ASN1_EXP(NEWPKI_PUB_REQUEST_BODY, d.cert, X509, NEWPKI_PUB_REQUEST_CERT),
	ASN1_EXP(NEWPKI_PUB_REQUEST_BODY, d.rev, NEWPKI_PUB_REQUEST_BODY_REV, NEWPKI_PUB_REQUEST_REV),
	ASN1_EXP(NEWPKI_PUB_REQUEST_BODY, d.crl, X509_CRL, NEWPKI_PUB_REQUEST_CRL),
} ASN1_CHOICE_END(NEWPKI_PUB_REQUEST_BODY)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PUB_REQUEST_BODY)

ASN1_SEQUENCE(NEWPKI_PUB_REQUEST) = {
	ASN1_SIMPLE(NEWPKI_PUB_REQUEST, ldap_uid, ASN1_UTF8STRING),
	ASN1_SEQUENCE_OF(NEWPKI_PUB_REQUEST, ParentCerts, X509),
	ASN1_SIMPLE(NEWPKI_PUB_REQUEST, object, ASN1_UTF8STRING),
	ASN1_SIMPLE(NEWPKI_PUB_REQUEST, body, NEWPKI_PUB_REQUEST_BODY),
}ASN1_SEQUENCE_END(NEWPKI_PUB_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PUB_REQUEST)



ASN1_CHOICE(NEWPKI_REQUEST) = {
	ASN1_EXP(NEWPKI_REQUEST, d.cert_request, NEWPKI_CERT_REQUEST, NEWPKI_REQUEST_TYPE_CERT),
	ASN1_EXP(NEWPKI_REQUEST, d.rev_request, NEWPKI_REV_REQUEST, NEWPKI_REQUEST_TYPE_REV),
	ASN1_EXP(NEWPKI_REQUEST, d.pub_request, NEWPKI_PUB_REQUEST, NEWPKI_REQUEST_TYPE_PUB),
} ASN1_CHOICE_END(NEWPKI_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_REQUEST)
IMPLEMENT_ASN1_ENCRYPT_SIGN(NEWPKI_REQUEST)


ASN1_SEQUENCE(CRYPTED_NEWPKI_REQUEST) = {
	ASN1_SIMPLE(CRYPTED_NEWPKI_REQUEST, sender, X509_PUBKEY),
	ASN1_SIMPLE(CRYPTED_NEWPKI_REQUEST, recipient, X509_PUBKEY),
	ASN1_SIMPLE(CRYPTED_NEWPKI_REQUEST, c_request, ASN1_ENCRYPT_SIGN),
}ASN1_SEQUENCE_END(CRYPTED_NEWPKI_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(CRYPTED_NEWPKI_REQUEST)
IMPLEMENT_ASN1_CPY_FUNCTIONS(CRYPTED_NEWPKI_REQUEST)
IMPLEMENT_PEM_CONVERT(CRYPTED_NEWPKI_REQUEST, "CRYPTED NEWPKI REQUEST")


ASN1_SEQUENCE(STORED_NEWPKI_REQUEST) = {
	ASN1_SIMPLE(STORED_NEWPKI_REQUEST, transactionID, ASN1_OCTET_STRING),
	ASN1_SIMPLE(STORED_NEWPKI_REQUEST, c_info, CRYPTED_NEWPKI_REQUEST),
}ASN1_SEQUENCE_END(STORED_NEWPKI_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(STORED_NEWPKI_REQUEST)
IMPLEMENT_ASN1_CPY_FUNCTIONS(STORED_NEWPKI_REQUEST)
IMPLEMENT_PEM_CONVERT(STORED_NEWPKI_REQUEST, "STORED NEWPKI REQUEST")


ASN1_SEQUENCE(STORED_NEWPKI_REQUESTS) = {
	ASN1_SEQUENCE_OF(STORED_NEWPKI_REQUESTS, requests, STORED_NEWPKI_REQUEST),
}ASN1_SEQUENCE_END(STORED_NEWPKI_REQUESTS)
IMPLEMENT_ASN1_FUNCTIONS(STORED_NEWPKI_REQUESTS)


ASN1_SEQUENCE(NEWPKI_CERT_RESPONSE) = {
	ASN1_SIMPLE(NEWPKI_CERT_RESPONSE, id, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_CERT_RESPONSE, status, ASN1_INTEGER),
	ASN1_EXP_OPT(NEWPKI_CERT_RESPONSE, certificate, X509, 0),
	ASN1_EXP_SEQUENCE_OF_OPT(NEWPKI_CERT_RESPONSE, ParentCerts, X509, 1),
	ASN1_EXP_SEQUENCE_OF_OPT(NEWPKI_CERT_RESPONSE, errors, ERROR_ENTRY, 2),
	ASN1_EXP_OPT(NEWPKI_CERT_RESPONSE, last_crl, X509_CRL, 3),
}ASN1_SEQUENCE_END(NEWPKI_CERT_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_CERT_RESPONSE)



ASN1_SEQUENCE(NEWPKI_REV_RESPONSE) = {
	ASN1_SIMPLE(NEWPKI_REV_RESPONSE, id, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_REV_RESPONSE, status, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_REV_RESPONSE, cert_status, ASN1_INTEGER),
	ASN1_EXP_SEQUENCE_OF_OPT(NEWPKI_REV_RESPONSE, errors, ERROR_ENTRY, 0),
	ASN1_EXP_OPT(NEWPKI_REV_RESPONSE, last_crl, X509_CRL, 1),
}ASN1_SEQUENCE_END(NEWPKI_REV_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_REV_RESPONSE)



ASN1_SEQUENCE(NEWPKI_PUB_RESPONSE) = {
	ASN1_SIMPLE(NEWPKI_PUB_RESPONSE, status, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PUB_RESPONSE, type, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PUB_RESPONSE, object, ASN1_UTF8STRING),
	ASN1_EXP_SEQUENCE_OF_OPT(NEWPKI_PUB_RESPONSE, errors, ERROR_ENTRY, 0),
}ASN1_SEQUENCE_END(NEWPKI_PUB_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PUB_RESPONSE)


ASN1_CHOICE(NEWPKI_RESPONSE) = {
	ASN1_EXP(NEWPKI_RESPONSE, d.cert_response, NEWPKI_CERT_RESPONSE, NEWPKI_RESPONSE_TYPE_CERT),
	ASN1_EXP(NEWPKI_RESPONSE, d.rev_response, NEWPKI_REV_RESPONSE, NEWPKI_RESPONSE_TYPE_REV),
	ASN1_EXP_SEQUENCE_OF(NEWPKI_RESPONSE, d.errors, ERROR_ENTRY, NEWPKI_RESPONSE_TYPE_ERR),
	ASN1_EXP(NEWPKI_RESPONSE, d.pub_response, NEWPKI_PUB_RESPONSE, NEWPKI_RESPONSE_TYPE_PUB),
} ASN1_CHOICE_END(NEWPKI_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_RESPONSE)
IMPLEMENT_ASN1_ENCRYPT_SIGN(NEWPKI_RESPONSE)


ASN1_SEQUENCE(CRYPTED_NEWPKI_RESPONSE) = {
	ASN1_SIMPLE(CRYPTED_NEWPKI_RESPONSE, sender, X509_PUBKEY),
	ASN1_SIMPLE(CRYPTED_NEWPKI_RESPONSE, recipient, X509_PUBKEY),
	ASN1_SIMPLE(CRYPTED_NEWPKI_RESPONSE, c_response, ASN1_ENCRYPT_SIGN),
}ASN1_SEQUENCE_END(CRYPTED_NEWPKI_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(CRYPTED_NEWPKI_RESPONSE)


ASN1_SEQUENCE(STORED_NEWPKI_RESPONSE) = {
	ASN1_SIMPLE(STORED_NEWPKI_RESPONSE, transactionID, ASN1_OCTET_STRING),
	ASN1_SIMPLE(STORED_NEWPKI_RESPONSE, c_info, CRYPTED_NEWPKI_RESPONSE),
}ASN1_SEQUENCE_END(STORED_NEWPKI_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(STORED_NEWPKI_RESPONSE)
IMPLEMENT_ASN1_CPY_FUNCTIONS(STORED_NEWPKI_RESPONSE)
IMPLEMENT_PEM_CONVERT(STORED_NEWPKI_RESPONSE, "STORED NEWPKI RESPONSE")

ASN1_SEQUENCE(STORED_NEWPKI_RESPONSES) = {
	ASN1_SEQUENCE_OF(STORED_NEWPKI_RESPONSES, responses, STORED_NEWPKI_RESPONSE),
}ASN1_SEQUENCE_END(STORED_NEWPKI_RESPONSES)
IMPLEMENT_ASN1_FUNCTIONS(STORED_NEWPKI_RESPONSES)


#define NEWPKI_OBJECTS_TYPE_REQ		0
#define NEWPKI_OBJECTS_TYPE_RESP	1

ASN1_CHOICE(WAITING_NEWPKI_OBJECT_BODY) = {
	ASN1_EXP(WAITING_NEWPKI_OBJECT_BODY, d.request, STORED_NEWPKI_REQUEST, NEWPKI_OBJECTS_TYPE_REQ),
	ASN1_EXP(WAITING_NEWPKI_OBJECT_BODY, d.response, STORED_NEWPKI_RESPONSE, NEWPKI_OBJECTS_TYPE_RESP),
} ASN1_CHOICE_END(WAITING_NEWPKI_OBJECT_BODY)
IMPLEMENT_ASN1_FUNCTIONS(WAITING_NEWPKI_OBJECT_BODY)


ASN1_SEQUENCE(TRANSACTION_IDS) = {
	ASN1_SEQUENCE_OF(TRANSACTION_IDS, transactionIds, ASN1_OCTET_STRING),
}ASN1_SEQUENCE_END(TRANSACTION_IDS)
IMPLEMENT_ASN1_FUNCTIONS(TRANSACTION_IDS)
IMPLEMENT_ASN1_CPY_FUNCTIONS(TRANSACTION_IDS)

ASN1_SEQUENCE(WAITING_NEWPKI_OBJECT) = {
	ASN1_SIMPLE(WAITING_NEWPKI_OBJECT, object, WAITING_NEWPKI_OBJECT_BODY),
	ASN1_SEQUENCE_OF(WAITING_NEWPKI_OBJECT, rep_path, X509_PUBKEY),
}ASN1_SEQUENCE_END(WAITING_NEWPKI_OBJECT)
IMPLEMENT_ASN1_FUNCTIONS(WAITING_NEWPKI_OBJECT)
IMPLEMENT_ASN1_CPY_FUNCTIONS(WAITING_NEWPKI_OBJECT)


ASN1_SEQUENCE(NEWPKI_PROFILE) = {
	ASN1_SIMPLE(NEWPKI_PROFILE, owner_group_serial, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE, dn, X509_NAME),
	ASN1_SIMPLE(NEWPKI_PROFILE, EE_Validation, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE, ldap_uid, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(NEWPKI_PROFILE)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PROFILE)
IMPLEMENT_PEM_CONVERT(NEWPKI_PROFILE, "NEWPKI PROFILE")


ASN1_SEQUENCE(NEWPKI_PROFILE_DATAS_CERT) = {
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS_CERT, Id, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS_CERT, state, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS_CERT, type, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS_CERT, ca_name, ASN1_UTF8STRING),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS_CERT, flags, ASN1_BIT_STRING),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS_CERT, admin_mail, ASN1_UTF8STRING),
	ASN1_EXP_OPT(NEWPKI_PROFILE_DATAS_CERT, cert, X509, 0),
	ASN1_EXP_OPT(NEWPKI_PROFILE_DATAS_CERT, error, ASN1_UTF8STRING, 1),
	ASN1_EXP_OPT(NEWPKI_PROFILE_DATAS_CERT, p12, PKCS12, 2),
}ASN1_SEQUENCE_END(NEWPKI_PROFILE_DATAS_CERT)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PROFILE_DATAS_CERT)


ASN1_SEQUENCE(NEWPKI_PROFILE_DATAS) = {
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS, State, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS, Id, ASN1_INTEGER),
	ASN1_SIMPLE(NEWPKI_PROFILE_DATAS, Profile, NEWPKI_PROFILE),
	ASN1_SEQUENCE_OF(NEWPKI_PROFILE_DATAS, Certs, NEWPKI_PROFILE_DATAS_CERT),
}ASN1_SEQUENCE_END(NEWPKI_PROFILE_DATAS)
IMPLEMENT_ASN1_FUNCTIONS(NEWPKI_PROFILE_DATAS)


ASN1_SEQUENCE(OBJECTS_LIST_REQUEST) = {
	ASN1_SIMPLE(OBJECTS_LIST_REQUEST, requests, TRANSACTION_IDS),
	ASN1_SIMPLE(OBJECTS_LIST_REQUEST, responses, TRANSACTION_IDS),
}ASN1_SEQUENCE_END(OBJECTS_LIST_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(OBJECTS_LIST_REQUEST)
IMPLEMENT_ASN1_CPY_FUNCTIONS(OBJECTS_LIST_REQUEST)

ASN1_SEQUENCE(OBJECTS_LIST_RESPONSE) = {
	ASN1_SIMPLE(OBJECTS_LIST_RESPONSE, requests, TRANSACTION_IDS),
	ASN1_SIMPLE(OBJECTS_LIST_RESPONSE, responses, TRANSACTION_IDS),
	ASN1_SEQUENCE_OF(OBJECTS_LIST_RESPONSE, objects, WAITING_NEWPKI_OBJECT),
}ASN1_SEQUENCE_END(OBJECTS_LIST_RESPONSE)
IMPLEMENT_ASN1_FUNCTIONS(OBJECTS_LIST_RESPONSE)
IMPLEMENT_ASN1_CPY_FUNCTIONS(OBJECTS_LIST_RESPONSE)

