/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "ASN1_entity.h"



ASN1_CHOICE(GEN_PRIVATE_KEY) = {
	ASN1_EXP(GEN_PRIVATE_KEY, d.keylen, ASN1_INTEGER, GEN_PRIVATE_KEY_TYPE_KEYLEN),
	ASN1_EXP(GEN_PRIVATE_KEY, d.keyid, ASN1_UTF8STRING, GEN_PRIVATE_KEY_TYPE_ENGINE),
} ASN1_CHOICE_END(GEN_PRIVATE_KEY)
IMPLEMENT_ASN1_FUNCTIONS(GEN_PRIVATE_KEY)

ASN1_SEQUENCE(ENTITY_ENTRY_INFO) = {
	ASN1_EXP(ENTITY_ENTRY_INFO, Name, ASN1_UTF8STRING, 0),
	ASN1_EXP(ENTITY_ENTRY_INFO, Type, ASN1_INTEGER, 1),
	ASN1_EXP_OPT(ENTITY_ENTRY_INFO, certificate, X509, 2),
	ASN1_SIMPLE(ENTITY_ENTRY_INFO, flags, ASN1_BIT_STRING),
}ASN1_SEQUENCE_END(ENTITY_ENTRY_INFO)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_ENTRY_INFO)
IMPLEMENT_ASN1_CPY_FUNCTIONS(ENTITY_ENTRY_INFO)


ASN1_SEQUENCE(PKI_CREATION_REQ) = {
	ASN1_SIMPLE(PKI_CREATION_REQ, Email, ASN1_UTF8STRING),
	ASN1_SIMPLE(PKI_CREATION_REQ, AdminEmail, ASN1_UTF8STRING),
	ASN1_SIMPLE(PKI_CREATION_REQ, validity, ASN1_INTEGER),
	ASN1_SIMPLE(PKI_CREATION_REQ, AdminPubKey, X509_PUBKEY),
	ASN1_SIMPLE(PKI_CREATION_REQ, dn, X509_NAME),
	ASN1_SIMPLE(PKI_CREATION_REQ, root_ca, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(PKI_CREATION_REQ, entities_ca, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(PKI_CREATION_REQ, users_ca, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(PKI_CREATION_REQ, ocsp_ca, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(PKI_CREATION_REQ, entity_key, GEN_PRIVATE_KEY),
}ASN1_SEQUENCE_END(PKI_CREATION_REQ)
IMPLEMENT_ASN1_FUNCTIONS(PKI_CREATION_REQ)


ASN1_SEQUENCE(PUB_CREATION_REQ) = {
	ASN1_SIMPLE(PUB_CREATION_REQ, entity_key, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(PUB_CREATION_REQ, ocsp_key, GEN_PRIVATE_KEY),
}ASN1_SEQUENCE_END(PUB_CREATION_REQ)
IMPLEMENT_ASN1_FUNCTIONS(PUB_CREATION_REQ)


ASN1_CHOICE(ENTITY_CREATION_DATAS) = {
	ASN1_EXP(ENTITY_CREATION_DATAS, d.entity_key, GEN_PRIVATE_KEY, ENTITY_TYPE_RA),
	ASN1_EXP(ENTITY_CREATION_DATAS, d.entity_key, GEN_PRIVATE_KEY, ENTITY_TYPE_CA),
	ASN1_EXP(ENTITY_CREATION_DATAS, d.entity_key, GEN_PRIVATE_KEY, ENTITY_TYPE_REPOSITORY),
	ASN1_EXP(ENTITY_CREATION_DATAS, d.pub_create, PUB_CREATION_REQ, ENTITY_TYPE_PUBLICATION),
	ASN1_EXP(ENTITY_CREATION_DATAS, d.entity_key, GEN_PRIVATE_KEY, ENTITY_TYPE_KEY_STORE),
	ASN1_EXP(ENTITY_CREATION_DATAS, d.pki_create, PKI_CREATION_REQ, ENTITY_TYPE_PKI),
} ASN1_CHOICE_END(ENTITY_CREATION_DATAS)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_CREATION_DATAS)


ASN1_SEQUENCE(ENTITY_CREATION_REQ) = {
	ASN1_SIMPLE(ENTITY_CREATION_REQ, Name, ASN1_UTF8STRING),
	ASN1_SIMPLE(ENTITY_CREATION_REQ, Datas, ENTITY_CREATION_DATAS),
}ASN1_SEQUENCE_END(ENTITY_CREATION_REQ)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_CREATION_REQ)


ASN1_SEQUENCE(PUB_ENTITY_CREATION_RESP) = {
	ASN1_SIMPLE(PUB_ENTITY_CREATION_RESP, entity_key, X509_PUBKEY),
	ASN1_SIMPLE(PUB_ENTITY_CREATION_RESP, ocsp_key, X509_PUBKEY),
}ASN1_SEQUENCE_END(PUB_ENTITY_CREATION_RESP)
IMPLEMENT_ASN1_FUNCTIONS(PUB_ENTITY_CREATION_RESP)


ASN1_CHOICE(ENTITY_CREATION_RESP) = {
	ASN1_EXP(ENTITY_CREATION_RESP, d.entity_pub_key, X509_PUBKEY, ENTITY_TYPE_RA),
	ASN1_EXP(ENTITY_CREATION_RESP, d.entity_pub_key, X509_PUBKEY, ENTITY_TYPE_CA),
	ASN1_EXP(ENTITY_CREATION_RESP, d.entity_pub_key, X509_PUBKEY, ENTITY_TYPE_REPOSITORY),
	ASN1_EXP(ENTITY_CREATION_RESP, d.pub_create, PUB_ENTITY_CREATION_RESP, ENTITY_TYPE_PUBLICATION),
	ASN1_EXP(ENTITY_CREATION_RESP, d.entity_pub_key, X509_PUBKEY, ENTITY_TYPE_KEY_STORE),
	ASN1_EXP_SEQUENCE_OF(ENTITY_CREATION_RESP, d.pki_create, X509, ENTITY_TYPE_PKI),
} ASN1_CHOICE_END(ENTITY_CREATION_RESP)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_CREATION_RESP)



ASN1_SEQUENCE(ENTITY_SIGNATURE_REQ_PUB) = {
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ_PUB, EntityPubKey, X509_PUBKEY),
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ_PUB, OcspPubKey, X509_PUBKEY),
}ASN1_SEQUENCE_END(ENTITY_SIGNATURE_REQ_PUB)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_REQ_PUB)



ASN1_SEQUENCE(ENTITY_SIGNATURE_REQ_REP) = {
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ_REP, EntityPubKey, X509_PUBKEY),
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ_REP, Address, ASN1_UTF8STRING),
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ_REP, Port, ASN1_INTEGER),
}ASN1_SEQUENCE_END(ENTITY_SIGNATURE_REQ_REP)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_REQ_REP)


ASN1_CHOICE(ENTITY_SIGNATURE_REQ_BODY) = {
	ASN1_EXP(ENTITY_SIGNATURE_REQ_BODY, d.EntityPubKey, X509_PUBKEY, ENTITY_TYPE_RA),
	ASN1_EXP(ENTITY_SIGNATURE_REQ_BODY, d.EntityPubKey, X509_PUBKEY, ENTITY_TYPE_CA),
	ASN1_EXP(ENTITY_SIGNATURE_REQ_BODY, d.sign_rep, ENTITY_SIGNATURE_REQ_REP, ENTITY_TYPE_REPOSITORY),
	ASN1_EXP(ENTITY_SIGNATURE_REQ_BODY, d.sign_pub, ENTITY_SIGNATURE_REQ_PUB, ENTITY_TYPE_PUBLICATION),
	ASN1_EXP(ENTITY_SIGNATURE_REQ_BODY, d.EntityPubKey, X509_PUBKEY, ENTITY_TYPE_KEY_STORE),
} ASN1_CHOICE_END(ENTITY_SIGNATURE_REQ_BODY)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_REQ_BODY)


ASN1_SEQUENCE(ENTITY_SIGNATURE_REQ) = {
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ, Name, ASN1_UTF8STRING),
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ, Email, ASN1_UTF8STRING),
	ASN1_SIMPLE(ENTITY_SIGNATURE_REQ, body, ENTITY_SIGNATURE_REQ_BODY),
}ASN1_SEQUENCE_END(ENTITY_SIGNATURE_REQ)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_REQ)
IMPLEMENT_PEM_CONVERT(ENTITY_SIGNATURE_REQ, "NEWPKI ENTITY SIGNATURE REQUEST");
	

ASN1_SEQUENCE(ENTITY_SIGNATURE_RESP_PUB) = {
	ASN1_SIMPLE(ENTITY_SIGNATURE_RESP_PUB, EntityCert, X509),
	ASN1_SIMPLE(ENTITY_SIGNATURE_RESP_PUB, OcspCert, X509),
}ASN1_SEQUENCE_END(ENTITY_SIGNATURE_RESP_PUB)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_RESP_PUB)



ASN1_CHOICE(ENTITY_SIGNATURE_RESP_BODY) = {
	ASN1_EXP(ENTITY_SIGNATURE_RESP_BODY, d.EntityCert, X509, ENTITY_TYPE_RA),
	ASN1_EXP(ENTITY_SIGNATURE_RESP_BODY, d.EntityCert, X509, ENTITY_TYPE_CA),
	ASN1_EXP(ENTITY_SIGNATURE_RESP_BODY, d.EntityCert, X509, ENTITY_TYPE_REPOSITORY),
	ASN1_EXP(ENTITY_SIGNATURE_RESP_BODY, d.sign_pub, ENTITY_SIGNATURE_RESP_PUB, ENTITY_TYPE_PUBLICATION),
	ASN1_EXP(ENTITY_SIGNATURE_RESP_BODY, d.EntityCert, X509, ENTITY_TYPE_KEY_STORE),
} ASN1_CHOICE_END(ENTITY_SIGNATURE_RESP_BODY)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_RESP_BODY)


ASN1_SEQUENCE(ENTITY_SIGNATURE_RESP) = {
	ASN1_SIMPLE(ENTITY_SIGNATURE_RESP, body, ENTITY_SIGNATURE_RESP_BODY),
	ASN1_SIMPLE(ENTITY_SIGNATURE_RESP, conf, ENTITY_CONF_CRYPTED),
	ASN1_SIMPLE(ENTITY_SIGNATURE_RESP, cas, INTERNAL_PKI_CA),
}ASN1_SEQUENCE_END(ENTITY_SIGNATURE_RESP)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_SIGNATURE_RESP)
IMPLEMENT_PEM_CONVERT(ENTITY_SIGNATURE_RESP, "NEWPKI ENTITY SIGNATURE RESPONSE");
IMPLEMENT_ASN1_ENCRYPT_SIGN(ENTITY_SIGNATURE_RESP)

ASN1_SEQUENCE(ENTITY_INIT_REQ) = {
	ASN1_SIMPLE(ENTITY_INIT_REQ, name, ASN1_UTF8STRING),
	ASN1_SIMPLE(ENTITY_INIT_REQ, sign_resp, ENTITY_SIGNATURE_RESP),
}ASN1_SEQUENCE_END(ENTITY_INIT_REQ)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_INIT_REQ)

ASN1_SEQUENCE(ENTITY_LINKS) = {
	ASN1_SIMPLE(ENTITY_LINKS, src, ENTITY_ENTRY_INFO),
	ASN1_SEQUENCE_OF(ENTITY_LINKS, dsts, ENTITY_ENTRY_INFO),
}ASN1_SEQUENCE_END(ENTITY_LINKS)
IMPLEMENT_ASN1_FUNCTIONS(ENTITY_LINKS)


ASN1_SEQUENCE(ENTITIES_LINKS) = {
	ASN1_SEQUENCE_OF(ENTITIES_LINKS, links, ENTITY_LINKS),
}ASN1_SEQUENCE_END(ENTITIES_LINKS)
IMPLEMENT_ASN1_FUNCTIONS(ENTITIES_LINKS)
IMPLEMENT_ASN1_CPY_FUNCTIONS(ENTITIES_LINKS)


ASN1_SEQUENCE(REQ_CREATE_ROOT_CA) = {
	ASN1_SIMPLE(REQ_CREATE_ROOT_CA, privKey, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(REQ_CREATE_ROOT_CA, dn, X509_NAME),
	ASN1_SIMPLE(REQ_CREATE_ROOT_CA, validity, ASN1_INTEGER),
	ASN1_SEQUENCE_OF(REQ_CREATE_ROOT_CA, extensions, EXTENSION_VALUE),
}ASN1_SEQUENCE_END(REQ_CREATE_ROOT_CA)
IMPLEMENT_ASN1_FUNCTIONS(REQ_CREATE_ROOT_CA)

ASN1_SEQUENCE(REQ_CREATE_CHILD_CA) = {
	ASN1_SIMPLE(REQ_CREATE_CHILD_CA, privKey, GEN_PRIVATE_KEY),
	ASN1_SIMPLE(REQ_CREATE_CHILD_CA, dn, X509_NAME),
}ASN1_SEQUENCE_END(REQ_CREATE_CHILD_CA)
IMPLEMENT_ASN1_FUNCTIONS(REQ_CREATE_CHILD_CA)


