/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "ASN1_err.h"


ASN1_SEQUENCE(ERROR_ENTRY) = {
	ASN1_SIMPLE(ERROR_ENTRY, line, ASN1_INTEGER),
	ASN1_SIMPLE(ERROR_ENTRY, file, ASN1_UTF8STRING),
	ASN1_SIMPLE(ERROR_ENTRY, lib, ASN1_INTEGER),
	ASN1_SIMPLE(ERROR_ENTRY, function, ASN1_INTEGER),
	ASN1_SIMPLE(ERROR_ENTRY, code, ASN1_INTEGER),
	ASN1_SIMPLE(ERROR_ENTRY, data, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(ERROR_ENTRY)
IMPLEMENT_ASN1_FUNCTIONS(ERROR_ENTRY)


#ifdef WIN32
	#define PATH_SEPARATOR '\\'
#else
	#define PATH_SEPARATOR '/'
#endif



STACK_OF(ERROR_ENTRY) * ERR_to_ERROR_ENTRIES()
{
	unsigned long lastError;
	const char *file,*data;
	int line,flags;
	char * ret_ErrString;
	char * ret_ErrLib;
	char * FileName;
	ERR_STATE * es;
	int i;
	STACK_OF(ERROR_ENTRY) * errors;
	ERROR_ENTRY * newError;

	errors = sk_new_null();
	if(!errors)
	{
		return NULL;
	}


	es = ERR_get_state();
	if(!es)
	{
		SKM_sk_pop_free(ERROR_ENTRY, errors, ERROR_ENTRY_free);
		return NULL;
	}


	data = NULL;

	//We do this not to erase the error from the queue
	for(i=es->bottom+1; i <= es->top; i++)
	{
		lastError = es->err_buffer[i];

		line = es->err_line[i];
		file = es->err_file[i];
		flags=es->err_data_flags[i];
		data=es->err_data[i];

		ret_ErrString = (char *)ERR_reason_error_string(lastError);
		if(!ret_ErrString)
		{
			if(data && *data && (flags & ERR_TXT_STRING) )
			{
				ret_ErrString = (char*)data;
				data = NULL;
			}
			else
				continue;
		}
		
		ret_ErrLib = (char*)ERR_lib_error_string(lastError);		
		if(!ret_ErrLib)
			ret_ErrLib = (char*)ERR_func_error_string(lastError);
		if(!ret_ErrLib)
			ret_ErrLib = "NewPKI";


		FileName = strrchr(file, PATH_SEPARATOR);
		if(!FileName)
			FileName = (char*)file;
		else
			FileName++;

		newError = ERROR_ENTRY_new();
		if(!newError) continue;

		ASN1_INTEGER_SET(&(newError->line), line);
		ASN1_STRING_SET(&(newError->file), file);
		ASN1_INTEGER_SET(&(newError->lib), ERR_GET_LIB(lastError));
		ASN1_INTEGER_SET(&(newError->function), ERR_GET_FUNC(lastError));
		ASN1_INTEGER_SET(&(newError->code), ERR_GET_REASON(lastError));
		if(data && (flags & ERR_TXT_STRING)) ASN1_STRING_SET(&(newError->data), data);
		SKM_sk_push(ERROR_ENTRY, errors, newError);
		data=NULL;
	}

	return errors;
}

