/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/


#ifndef ASN1_ERR_H
#define ASN1_ERR_H

#include <openssl/x509.h>
#include <openssl/asn1.h>
#include <openssl/asn1t.h>
#include <openssl/x509v3.h>
#include <openssl/evp.h>
#include <openssl/pem.h>

#include "my_asn1.h"


/*!
  This structure represents an error
*/
typedef struct st_ERROR_ENTRY
{
	ASN1_INTEGER *line;			//!< The line
	ASN1_UTF8STRING * file;		//!< The file
	ASN1_INTEGER * lib;			//!< The lib
	ASN1_INTEGER * function;	//!< The function
	ASN1_INTEGER * code;		//!< The error code
	ASN1_UTF8STRING * data;		//!< Some additional datas
}ERROR_ENTRY;
DECLARE_ASN1_FUNCTIONS(ERROR_ENTRY)


/*! \brief This function converts OpenSSL's errors stack into a list of errors.
 *  \return The list of errors.
 */
STACK_OF(ERROR_ENTRY) * ERR_to_ERROR_ENTRIES();

#endif
