/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "ASN1_logs.h"
#include "../lbintl.h"


ASN1_SEQUENCE(LOG_ENTRY_BODY) = {
	ASN1_SIMPLE(LOG_ENTRY_BODY, log_status, ASN1_INTEGER),
	ASN1_SIMPLE(LOG_ENTRY_BODY, log_type, ASN1_INTEGER),
	ASN1_SIMPLE(LOG_ENTRY_BODY, user, ASN1_UTF8STRING),
	ASN1_SIMPLE(LOG_ENTRY_BODY, object_name, ASN1_UTF8STRING),
	ASN1_SIMPLE(LOG_ENTRY_BODY, error, ASN1_UTF8STRING),
	ASN1_SIMPLE(LOG_ENTRY_BODY, log_date, ASN1_INTEGER),
}ASN1_SEQUENCE_END(LOG_ENTRY_BODY)
IMPLEMENT_ASN1_FUNCTIONS(LOG_ENTRY_BODY)


ASN1_SEQUENCE(LOG_ENTRY_SIG) = {
	ASN1_SIMPLE(LOG_ENTRY_SIG, signature, ASN1_BIT_STRING),
	ASN1_SIMPLE(LOG_ENTRY_SIG, sig_alg, X509_ALGOR),
}ASN1_SEQUENCE_END(LOG_ENTRY_SIG)
IMPLEMENT_ASN1_FUNCTIONS(LOG_ENTRY_SIG)
IMPLEMENT_PEM_CONVERT(LOG_ENTRY_SIG, "NEWPKI LOG SIGNATURE");


ASN1_SEQUENCE(LOG_ENTRY) = {
	ASN1_SIMPLE(LOG_ENTRY, log_id, ASN1_INTEGER),
	ASN1_SIMPLE(LOG_ENTRY, body, LOG_ENTRY_BODY),
	ASN1_SIMPLE(LOG_ENTRY, sig, LOG_ENTRY_SIG),
}ASN1_SEQUENCE_END(LOG_ENTRY)
IMPLEMENT_ASN1_FUNCTIONS(LOG_ENTRY)


ASN1_SEQUENCE(ADMIN_REQ_ENUM_LOGS) = {
	ASN1_SIMPLE(ADMIN_REQ_ENUM_LOGS, index, ASN1_INTEGER),
	ASN1_SIMPLE(ADMIN_REQ_ENUM_LOGS, max, ASN1_INTEGER),
	ASN1_EXP_OPT(ADMIN_REQ_ENUM_LOGS, object, ASN1_UTF8STRING, 0),
	ASN1_EXP_OPT(ADMIN_REQ_ENUM_LOGS, type, ASN1_INTEGER, 1),
	ASN1_EXP_OPT(ADMIN_REQ_ENUM_LOGS, date_from, ASN1_INTEGER, 2),
	ASN1_EXP_OPT(ADMIN_REQ_ENUM_LOGS, date_to, ASN1_INTEGER, 3),
	ASN1_EXP_OPT(ADMIN_REQ_ENUM_LOGS, status, ASN1_INTEGER, 4),
	ASN1_EXP_OPT(ADMIN_REQ_ENUM_LOGS, user, ASN1_UTF8STRING, 5),
}ASN1_SEQUENCE_END(ADMIN_REQ_ENUM_LOGS)
IMPLEMENT_ASN1_FUNCTIONS(ADMIN_REQ_ENUM_LOGS)


typedef struct md_LOG_MESSAGE_STATUS_ENTRY
{
	LOG_MESSAGE_STATUS uID;
	char * Text;
}LOG_MESSAGE_STATUS_ENTRY;

typedef struct md_LOG_MESSAGE_ENTRY
{
	LOG_MESSAGE_TYPE uID;
	char * Text;
}LOG_MESSAGE_ENTRY;


LOG_MESSAGE_STATUS_ENTRY LogMessagesStatus[]={
	{ LOG_STATUS_TYPE_REQUEST,		_lb("Request") }, 
	{ LOG_STATUS_TYPE_SUCCESS,		_lb("Success") }, 
	{ LOG_STATUS_TYPE_FAILURE,		_lb("Failure") }, 
	{ LOG_STATUS_TYPE_FAILURE,		NULL }
};


LOG_MESSAGE_ENTRY LogMessages[]={
	{ LOG_MESSAGE_TYPE_USER_LOGIN,				_lb("User login") }, 
	{ LOG_MESSAGE_TYPE_USER_ADD,				_lb("User creation") }, 
	{ LOG_MESSAGE_TYPE_USER_MOD,				_lb("User modification") }, 
	{ LOG_MESSAGE_TYPE_PWD_MOD,					_lb("Password modification") }, 
	{ LOG_MESSAGE_TYPE_ENTITY_ADD,				_lb("Entity creation") }, 
	{ LOG_MESSAGE_TYPE_ENTITY_DEL,				_lb("Entity deletion") }, 
	{ LOG_MESSAGE_TYPE_SIGN_ENTITY,				_lb("Signature of a entity certificate request") }, 
	{ LOG_MESSAGE_TYPE_ENTITY_REVOCATION,		_lb("Revokation of an entity") }, 
	{ LOG_MESSAGE_TYPE_USER_REVOCATION,			_lb("Revokation of an user") }, 
	{ LOG_MESSAGE_TYPE_CONFIG_PUSH,				_lb("PKI Configuration push to repository") }, 
	{ LOG_MESSAGE_TYPE_SET_OFFLINE,				_lb("Toggle the PKI offline") }, 
	{ LOG_MESSAGE_TYPE_GET_CONFIGURATION,		_lb("Get PKI configuration") }, 
	{ LOG_MESSAGE_TYPE_CONFIG_IMPORT,			_lb("Importing PKI configuration") },
	{ LOG_MESSAGE_TYPE_ENTITY_CONFIG_GET,		_lb("Retrieval of its configuration by an entity") },
	{ LOG_MESSAGE_TYPE_ENTITY_INIT,				_lb("Entity initialization") },
	{ LOG_MESSAGE_TYPE_SET_REPOSITORIES,		_lb("Repositories info modification") },
	{ LOG_MESSAGE_TYPE_ENTITY_LOAD,				_lb("Entity loading") },
	{ LOG_MESSAGE_TYPE_LOCK_RESOURCE,			_lb("Resource locking") },
	{ LOG_MESSAGE_TYPE_UNLOCK_RESOURCE,			_lb("Resource unlocking") },
	{ LOG_MESSAGE_TYPE_SET_ENTITY_ACL,			_lb("Modification of an entity ACLs") },
	{ LOG_MESSAGE_TYPE_SET_ENTITY_MAIL_CONF,	_lb("Modification of an entity mail configuration") },
	{ LOG_MESSAGE_TYPE_SET_ENTITY_CONF,			_lb("Modification of an entity configuration") },
	{ LOG_MESSAGE_TYPE_SEND_ADMIN_MAIL,			_lb("Sending an admin notification mail") },
	{ LOG_MESSAGE_TYPE_SEND_MAIL,				_lb("Sending an user notification mail") },
	{ LOG_MESSAGE_TYPE_ADD_ADMIN_MAIL_QUEUE,	_lb("Adding an admin notification mail to queue") },
	{ LOG_MESSAGE_TYPE_ADD_MAIL_QUEUE,			_lb("Adding an user notification mail to queue") },
	{ LOG_MESSAGE_TYPE_SET_ENTITY_AUDITS,		_lb("Modification of an entity audits") },
	{ LOG_MESSAGE_TYPE_SET_ENTITIES_LINKS,		_lb("Modification of the entities links") },
	{ LOG_MESSAGE_TYPE_ENTITY_GET_MY_CONF,		_lb("Retrieval of my configuration from a repository") },
	{ LOG_MESSAGE_TYPE_OBJECTS_PUSH,			_lb("Objects push to repository") },
	{ LOG_MESSAGE_TYPE_OBJECTS_IMPORT,			_lb("Importing objects") },
	{ LOG_MESSAGE_TYPE_CREATE_ROOT_CA,			_lb("Creating ROOT CA") },
	{ LOG_MESSAGE_TYPE_GET_MY_REQS,				_lb("Entity retreiving its requests") },
	{ LOG_MESSAGE_TYPE_GET_MY_RESPS,			_lb("Entity retreiving its responses") },
	{ LOG_MESSAGE_TYPE_SIGN_CSR,				_lb("CSR signature") },
	{ LOG_MESSAGE_TYPE_GEN_CRL,					_lb("CRL generation") },
	{ LOG_MESSAGE_TYPE_CREATE_CHILD_CA,			_lb("Creating Child CA") },
	{ LOG_MESSAGE_TYPE_IMPORT_CHILD_CA_CERT,	_lb("Importing Child CA cert") },
	{ LOG_MESSAGE_TYPE_CERT_REVOCATION,			_lb("Revokation of a certificate") }, 
	{ LOG_MESSAGE_TYPE_SET_GROUPS,				_lb("Modification of the groups") }, 
	{ LOG_MESSAGE_TYPE_ADD_GROUP,				_lb("Adding a group") }, 
	{ LOG_MESSAGE_TYPE_DEL_GROUP,				_lb("Deleting a group") }, 
	{ LOG_MESSAGE_TYPE_REN_GROUP,				_lb("Renaming a group") }, 
	{ LOG_MESSAGE_TYPE_GET_LOCAL_CONF,			_lb("User requests local conf") }, 
	{ LOG_MESSAGE_TYPE_IMPORT_PROFILE,			_lb("Importing a new profile") }, 
	{ LOG_MESSAGE_TYPE_INSERT_REQ,				_lb("Entity publishing a request") }, 
	{ LOG_MESSAGE_TYPE_INSERT_RESP,				_lb("Entity publishing a response") }, 
	{ LOG_MESSAGE_TYPE_CERT_REQUEST,			_lb("New certificate request for a profile") }, 
	{ LOG_MESSAGE_TYPE_DELETE_PKCS12,			_lb("Delete PKCS#12") },
	{ LOG_MESSAGE_TYPE_NEW_REQUET,				_lb("Received new request") },
	{ LOG_MESSAGE_TYPE_CA_CERT,					_lb("Certification request") },
	{ LOG_MESSAGE_TYPE_CA_REV,					_lb("Revokation request") },
	{ LOG_MESSAGE_TYPE_LOADING_PUB_METH,		_lb("Loading publication method") },
	{ LOG_MESSAGE_TYPE_CERT_PUBLICATION,		_lb("Certificate publication request") },
	{ LOG_MESSAGE_TYPE_REV_PUBLICATION,			_lb("Revocation publication request") },
	{ LOG_MESSAGE_TYPE_CRL_PUBLICATION,			_lb("CRL publication request") },
	{ LOG_MESSAGE_TYPE_LDAP_CONNECTION,			_lb("LDAP connection") },
	{ LOG_MESSAGE_TYPE_LDAP_AUTO_SYNCH_START,	_lb("LDAP auto synchronization start") },
	{ LOG_MESSAGE_TYPE_CERT_REV_RESP,			_lb("Revocation response") },
	{ LOG_MESSAGE_TYPE_CERT_SIG_RESP,			_lb("Signature response") },
	{ LOG_MESSAGE_TYPE_CERT_RNW_RESP,			_lb("Renewal response") },
	{ LOG_MESSAGE_TYPE_LDAP_AUTO_SYNCH_IMPRT,	_lb("LDAP auto synchronization profile importation") },
	{ LOG_MESSAGE_TYPE_OCSP_START,				_lb("Starting OCSP Responder") },
	{ LOG_MESSAGE_TYPE_OCSP_STOP,				_lb("Stopping OCSP Responder") },
	{ LOG_MESSAGE_TYPE_OCSP_REQUEST,			_lb("OCSP Request") },
	{ LOG_MESSAGE_TYPE_ADMIN_REQUEST,			_lb("ADMIN Request") },
	{ LOG_MESSAGE_TYPE_USER_LOGIN,				NULL }
};



char * ASN1_logs_get_TypeString(LOG_MESSAGE_TYPE log_type)
{
	for(int i=0; LogMessages[i].Text; i++)
	{
		if(LogMessages[i].uID == log_type) return LogMessages[i].Text;
	}
	return _lb("Unknown");
}

char * ASN1_logs_get_StatusString(LOG_MESSAGE_STATUS log_status)
{
	for(int i=0; LogMessagesStatus[i].Text; i++)
	{
		if(LogMessagesStatus[i].uID == log_status) return LogMessagesStatus[i].Text;
	}

	return _lb("Unknown");
}

