/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include "ASN1_req.h"


ASN1_CHOICE(ADMIN_REQUEST_BODY) = {
	ASN1_EXP(ADMIN_REQUEST_BODY, d.login, ADMIN_REQ_LOGIN, ADMIN_REQ_TYPE_LOGIN),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.cre_entity, ENTITY_CREATION_REQ, ADMIN_REQ_TYPE_CREATE_ENTITY),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.del_entity, ASN1_UTF8STRING, ADMIN_REQ_TYPE_DELETE_ENTITY),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_ENUM_ENTITIES),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.cre_user, USER_INFO, ADMIN_REQ_TYPE_CREATE_USER),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_ENUM_USERS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.upd_user, USER_ENTRY, ADMIN_REQ_TYPE_UPDATE_USER),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.enum_logs, ADMIN_REQ_ENUM_LOGS, ADMIN_REQ_TYPE_ENUM_LOGS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.chg_user_pwd, CHANGE_USER_PASSWD, ADMIN_REQ_TYPE_CHG_USER_PASSWD),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.chg_pwd, CHANGE_PASSWD, ADMIN_REQ_TYPE_CHG_PASSWD),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.sign_entity, ENTITY_SIGNATURE_REQ, ADMIN_REQ_TYPE_SIGN_ENTITY),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.create_pki_user, CREATE_PKI_USER, ADMIN_REQ_TYPE_CREATE_PKI_USER),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.serial, ASN1_INTEGER, ADMIN_REQ_TYPE_REVOKE_USER),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.serial, ASN1_INTEGER, ADMIN_REQ_TYPE_REVOKE_ENTITY),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.conf, EXPORTED_PKI_CONF, ADMIN_REQ_TYPE_PUSH_CONFIG),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.offline, ASN1_INTEGER, ADMIN_REQ_TYPE_SET_OFFLINE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_OFFLINE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_MY_ACL),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_CONFIG),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_MY_CONFIG),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.init_entity, ENTITY_INIT_REQ, ADMIN_REQ_TYPE_INIT_ENTITY),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_cert, X509, ADMIN_REQ_TYPE_GET_ENTITY_CONF),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_REPOSITORIES),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.repositories, REPS_INFO, ADMIN_REQ_TYPE_SET_REPOSITORIES),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_LOGS_TYPE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.resource_name, ASN1_UTF8STRING, ADMIN_REQ_TYPE_LOCK_RESOURCE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.resource_name, ASN1_UTF8STRING, ADMIN_REQ_TYPE_UNLOCK_RESOURCE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_cert, X509, ADMIN_REQ_TYPE_GET_ENTITY_ACL),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_cert, X509, ADMIN_REQ_TYPE_GET_ENTITY_MAIL_CONF),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_acl, SET_ENTITY_ACL, ADMIN_REQ_TYPE_SET_ENTITY_ACL),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_mail_conf, SET_ENTITY_MAIL_CONF, ADMIN_REQ_TYPE_SET_ENTITY_MAIL_CONF),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_conf, SET_ENTITY_CONF, ADMIN_REQ_TYPE_SET_ENTITY_CONF),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.admin_mail, MAIL_INFO, ADMIN_REQ_TYPE_SEND_ADMIN_MAIL),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_cert, X509, ADMIN_REQ_TYPE_GET_ENTITY_AUDITS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_audits, SET_ENTITY_AUDITS, ADMIN_REQ_TYPE_SET_ENTITY_AUDITS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entity_type, ASN1_INTEGER, ADMIN_REQ_TYPE_GET_ENTITY_LOGS_TYPE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_ENTITIES_LINKS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.entities_links, ENTITIES_LINKS, ADMIN_REQ_TYPE_SET_ENTITIES_LINKS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.waiting_obj, WAITING_NEWPKI_OBJECT, ADMIN_REQ_TYPE_PUSH_NEWPKI_OBJECTS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.create_root_ca, REQ_CREATE_ROOT_CA, ADMIN_REQ_TYPE_CREATE_ROOT_CA),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_CA_STATUS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.transactions_ids, TRANSACTION_IDS, ADMIN_REQ_TYPE_GET_MY_REQS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.enum_objects, ENUM_OBJECTS, ADMIN_REQ_TYPE_ENUM_CERTS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.enum_objects, ENUM_OBJECTS, ADMIN_REQ_TYPE_ENUM_CRLS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.sign_csr, SIGN_CSR, ADMIN_REQ_TYPE_SIGN_CSR),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.create_child_ca, REQ_CREATE_CHILD_CA, ADMIN_REQ_TYPE_CREATE_CHILD_CA),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.p7b, PKCS7, ADMIN_REQ_TYPE_IMPORT_CHILD_CA_CERT),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.serial, ASN1_INTEGER, ADMIN_REQ_TYPE_REVOKE_CERT),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_GROUPS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.groups, USERS_GROUPS, ADMIN_REQ_TYPE_SET_GROUPS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.group_name, ASN1_UTF8STRING, ADMIN_REQ_TYPE_ADD_GROUP),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.group_id, ASN1_INTEGER, ADMIN_REQ_TYPE_DEL_GROUP),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.rename_group, RENAME_GROUP, ADMIN_REQ_TYPE_REN_GROUP),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_CHECK_LOGS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.profile, NEWPKI_PROFILE, ADMIN_REQ_TYPE_IMPORT_PROFILE),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_LOCAL_CONF),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.new_req, CRYPTED_NEWPKI_REQUEST, ADMIN_REQ_TYPE_PUSH_REQ),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.new_resp, STORED_NEWPKI_RESPONSE, ADMIN_REQ_TYPE_PUSH_RESP),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.transactions_ids, TRANSACTION_IDS, ADMIN_REQ_TYPE_GET_MY_RESPS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.enum_objects, ENUM_OBJECTS, ADMIN_REQ_TYPE_ENUM_PROFILES),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.cert_request, REQUEST_CERT, ADMIN_REQ_TYPE_REQUEST_CERT),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GENERATE_CRL),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.known_objects, OBJECTS_LIST_REQUEST, ADMIN_REQ_TYPE_SYNCH_OBJECTS),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.other, ASN1_NULL, ADMIN_REQ_TYPE_GET_CA_CERT),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.serial, ASN1_INTEGER, ADMIN_REQ_TYPE_DELETE_PKCS12),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.enum_logs, ADMIN_REQ_ENUM_LOGS, ADMIN_REQ_TYPE_GET_LOGS_COUNT),
	ASN1_EXP(ADMIN_REQUEST_BODY, d.ldap_search, ASN1_UTF8STRING, ADMIN_REQ_TYPE_SEARCH_LDAP),
} ASN1_CHOICE_END(ADMIN_REQUEST_BODY)
IMPLEMENT_ASN1_FUNCTIONS(ADMIN_REQUEST_BODY)


ASN1_SEQUENCE(ADMIN_REQUEST) = {
	ASN1_SIMPLE(ADMIN_REQUEST, body, ADMIN_REQUEST_BODY),
}ASN1_SEQUENCE_END(ADMIN_REQUEST)
IMPLEMENT_ASN1_FUNCTIONS(ADMIN_REQUEST)


ASN1_SEQUENCE(ADMIN_REQ_LOGIN) = {
	ASN1_SIMPLE(ADMIN_REQ_LOGIN, entity, ASN1_UTF8STRING),
	ASN1_SIMPLE(ADMIN_REQ_LOGIN, username, ASN1_UTF8STRING),
	ASN1_SIMPLE(ADMIN_REQ_LOGIN, password, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(ADMIN_REQ_LOGIN)
IMPLEMENT_ASN1_FUNCTIONS(ADMIN_REQ_LOGIN)


ASN1_SEQUENCE(SET_ENTITY_MAIL_CONF) = {
	ASN1_SIMPLE(SET_ENTITY_MAIL_CONF, entity_cert, X509),
	ASN1_SIMPLE(SET_ENTITY_MAIL_CONF, mail_conf, EMAIL_CONF),
}ASN1_SEQUENCE_END(SET_ENTITY_MAIL_CONF)
IMPLEMENT_ASN1_FUNCTIONS(SET_ENTITY_MAIL_CONF)


ASN1_SEQUENCE(SET_ENTITY_ACL) = {
	ASN1_SIMPLE(SET_ENTITY_ACL, entity_cert, X509),
	ASN1_SEQUENCE_OF(SET_ENTITY_ACL, acl_entries, ACL_ENTRY),
}ASN1_SEQUENCE_END(SET_ENTITY_ACL)
IMPLEMENT_ASN1_FUNCTIONS(SET_ENTITY_ACL)


ASN1_SEQUENCE(SET_ENTITY_CONF) = {
	ASN1_SIMPLE(SET_ENTITY_CONF, entity_cert, X509),
	ASN1_SIMPLE(SET_ENTITY_CONF, conf, ENTITY_CONF_BODY),
}ASN1_SEQUENCE_END(SET_ENTITY_CONF)
IMPLEMENT_ASN1_FUNCTIONS(SET_ENTITY_CONF)


ASN1_SEQUENCE(MAIL_INFO) = {
	ASN1_SIMPLE(MAIL_INFO, Subject, ASN1_UTF8STRING),
	ASN1_SIMPLE(MAIL_INFO, Body, ASN1_UTF8STRING),
	ASN1_SIMPLE(MAIL_INFO, SignMail, ASN1_INTEGER),
	ASN1_SIMPLE(MAIL_INFO, Attach, ASN1_BIT_STRING),
	ASN1_SIMPLE(MAIL_INFO, AttachName, ASN1_UTF8STRING),
	ASN1_SIMPLE(MAIL_INFO, AttachType, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(MAIL_INFO)
IMPLEMENT_ASN1_FUNCTIONS(MAIL_INFO)
IMPLEMENT_ASN1_CPY_FUNCTIONS(MAIL_INFO)


ASN1_SEQUENCE(SET_ENTITY_AUDITS) = {
	ASN1_SIMPLE(SET_ENTITY_AUDITS, entity_cert, X509),
	ASN1_SIMPLE(SET_ENTITY_AUDITS, audits, ENTITY_AUDIT),
}ASN1_SEQUENCE_END(SET_ENTITY_AUDITS)
IMPLEMENT_ASN1_FUNCTIONS(SET_ENTITY_AUDITS)


ASN1_SEQUENCE(ENUM_OBJECTS) = {
	ASN1_SIMPLE(ENUM_OBJECTS, state, ASN1_INTEGER),
	ASN1_SIMPLE(ENUM_OBJECTS, index, ASN1_INTEGER),
	ASN1_SIMPLE(ENUM_OBJECTS, num, ASN1_INTEGER),
}ASN1_SEQUENCE_END(ENUM_OBJECTS)
IMPLEMENT_ASN1_FUNCTIONS(ENUM_OBJECTS)


ASN1_SEQUENCE(SIGN_CSR) = {
	ASN1_SIMPLE(SIGN_CSR, days, ASN1_INTEGER),
	ASN1_SIMPLE(SIGN_CSR, request, X509_REQ),
}ASN1_SEQUENCE_END(SIGN_CSR)
IMPLEMENT_ASN1_FUNCTIONS(SIGN_CSR)

ASN1_SEQUENCE(CREATE_PKI_USER) = {
	ASN1_SIMPLE(CREATE_PKI_USER, pkiadmin, ASN1_INTEGER),
	ASN1_SIMPLE(CREATE_PKI_USER, csr, X509_REQ),
}ASN1_SEQUENCE_END(CREATE_PKI_USER)
IMPLEMENT_ASN1_FUNCTIONS(CREATE_PKI_USER)


ASN1_SEQUENCE(RENAME_GROUP) = {
	ASN1_SIMPLE(RENAME_GROUP, id, ASN1_INTEGER),
	ASN1_SIMPLE(RENAME_GROUP, name, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(RENAME_GROUP)
IMPLEMENT_ASN1_FUNCTIONS(RENAME_GROUP)



ASN1_SEQUENCE(REQUEST_CERT_BODY_P12) = {
	ASN1_SIMPLE(REQUEST_CERT_BODY_P12, keylen, ASN1_INTEGER),
	ASN1_SIMPLE(REQUEST_CERT_BODY_P12, passwd, ASN1_UTF8STRING),
}ASN1_SEQUENCE_END(REQUEST_CERT_BODY_P12)
IMPLEMENT_ASN1_FUNCTIONS(REQUEST_CERT_BODY_P12)


ASN1_CHOICE(REQUEST_CERT_BODY) = {
	ASN1_EXP(REQUEST_CERT_BODY, d.pkcs10, X509_REQ, REQUEST_CERT_BODY_PKCS10),
	ASN1_EXP(REQUEST_CERT_BODY, d.p12_privdatas, REQUEST_CERT_BODY_P12, REQUEST_CERT_BODY_PKCS12),
} ASN1_CHOICE_END(REQUEST_CERT_BODY)
IMPLEMENT_ASN1_FUNCTIONS(REQUEST_CERT_BODY)

ASN1_SEQUENCE(REQUEST_CERT) = {
	ASN1_SIMPLE(REQUEST_CERT, profile_id, ASN1_INTEGER),
	ASN1_SIMPLE(REQUEST_CERT, ca_name, ASN1_UTF8STRING),
	ASN1_SIMPLE(REQUEST_CERT, request, REQUEST_CERT_BODY),
	ASN1_SIMPLE(REQUEST_CERT, validity, ASN1_INTEGER),
	ASN1_SIMPLE(REQUEST_CERT, type, ASN1_INTEGER),
	ASN1_SIMPLE(REQUEST_CERT, flags, ASN1_BIT_STRING),
}ASN1_SEQUENCE_END(REQUEST_CERT)
IMPLEMENT_ASN1_FUNCTIONS(REQUEST_CERT)
