/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/


#ifndef ASN1_USER_H
#define ASN1_USER_H

#include <openssl/x509.h>
#include <openssl/asn1.h>
#include <openssl/asn1t.h>
#include <openssl/x509v3.h>
#include <openssl/evp.h>
#include <openssl/pem.h>

#include "my_asn1.h"

#define USER_TYPE_SERVER		1		//!< The user that just logged in is a Server user
#define USER_TYPE_PKI			2		//!< The user that just logged in is a PKI user
#define USER_TYPE_CA			3		//!< The user that just logged in is a CA user
#define USER_TYPE_RA			4		//!< The user that just logged in is a RA user
#define USER_TYPE_REPOSITORY	5		//!< The user that just logged in is a Repository user
#define USER_TYPE_KEYSTORE		6		//!< The user that just logged in is a KeyStore user
#define USER_TYPE_PUBLICATION	7		//!< The user that just logged in is a Publication user


/*!
	This enumeration defines the possible flags for a user
*/
typedef enum USER_FLAGS
{
	USER_FLAGS_EXTERNAL_AUTH		= 1		//!< The authentication should be processed by an external module
};


/*!
	This structure defines a Server user personal info
*/
typedef struct st_USER_INFO
{
	ASN1_UTF8STRING * Name;			//!< The user name
	ASN1_INTEGER * Activated;		//!< The user activation status
	ASN1_INTEGER * Flags;			//!< The user flags
}USER_INFO;
DECLARE_ASN1_FUNCTIONS(USER_INFO)

/*!
	This structure defines a Server user
*/
typedef struct st_USER_ENTRY
{
	ASN1_INTEGER * user_id;			//!< The user id
	USER_INFO * user_info;			//!< The user information
}USER_ENTRY;
DECLARE_ASN1_FUNCTIONS(USER_ENTRY)

/*!
	This structure defines the certificate for a PKI user
*/
typedef struct st_USER_ENTRY_CERT
{
	X509 * user_cert;				//!< Its certificate
}USER_ENTRY_CERT;
DECLARE_ASN1_FUNCTIONS(USER_ENTRY_CERT)


/*!
	This structure defines a password change request for a server user
*/
typedef struct st_CHANGE_USER_PASSWD
{
	ASN1_INTEGER * user_id;			//!< The user id
	ASN1_UTF8STRING * Password;		//!< The new password
}CHANGE_USER_PASSWD;
DECLARE_ASN1_FUNCTIONS(CHANGE_USER_PASSWD)


/*!
	This structure defines a password change request by a server user
*/
typedef struct st_CHANGE_PASSWD
{
	ASN1_UTF8STRING * Password;		//!< The new password
}CHANGE_PASSWD;
DECLARE_ASN1_FUNCTIONS(CHANGE_PASSWD)


#endif
