/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/


#include "FileLog.h"


char FileName[250]={0};
LOG_LEVEL debuglevel=LOG_LEVEL_NONE;


void SetLogFile(const char * File)
{
	strcpy(FileName, File);
}

void SetDebugLevel(LOG_LEVEL level)
{
	debuglevel = level;
}

void LogInsertDelimiter(char * Delimiter)
{
	FILE * fd;

	if(!*FileName) return;

	#ifdef WIN32
		#define DELIM_PATT "%s\r\n"
	#else
		#define DELIM_PATT "%s\n"
	#endif

	fd = fopen(FileName, "ab");
	if(!fd)
	{
		return;
	}

	fprintf(fd, DELIM_PATT, Delimiter);

	#undef DELIM_PATT

	fclose(fd);
}

void LogInFile(char * prog, char * pattern, ...)
{
	FILE * fd;
	va_list marker;
	time_t timet;
	struct tm * tmt;
	char * strtime;


	if(!*FileName) return;

	time(&timet);
	tmt = localtime(&timet);
	strtime = asctime(tmt);
	if(!strtime) return;
	strtime[strlen(strtime)-1] = 0;

	fd = fopen(FileName, "ab");
	if(!fd)
	{
		return;
	}
	
	fprintf(fd, "##### {%s} > %s# ", prog, strtime);
	va_start( marker, pattern );
	vfprintf(fd, pattern, marker);
	va_end( marker );

	#ifdef WIN32
		#define SEP "\r\n"
	#else
		#define SEP "\n"
	#endif
	fprintf(fd, SEP);
	#undef SEP

	fclose(fd);
}

void NewpkiDebug(LOG_LEVEL level, const char * prog, const char * pattern, ...)
{
	FILE * fd;
	va_list marker;
	time_t timet;
	struct tm * tmt;
	char * strtime;

	if(debuglevel == LOG_LEVEL_NONE || debuglevel < level || !*FileName)
	{
		return;
	}

	time(&timet);
	tmt = localtime(&timet);
	strtime = asctime(tmt);
	if(!strtime) return;
	strtime[strlen(strtime)-1] = 0;

	fd = fopen(FileName, "ab");
	if(!fd)
	{
		return;
	}
	
	fprintf(fd, "##### Debug(%d) {%s} > %s# ", level, prog, strtime);
	va_start( marker, pattern );
	vfprintf(fd, pattern, marker);
	va_end( marker );

	#ifdef WIN32
		#define SEP "\r\n"
	#else
		#define SEP "\n"
	#endif
	fprintf(fd, SEP);
	#undef SEP

	fclose(fd);
}

