/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/


// PKI_EXT.cpp: implementation of the PKI_EXT class.
//
//////////////////////////////////////////////////////////////////////

#include "PKI_EXT.h"
#include <ctype.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


PKI_EXT::PKI_EXT()
{
}

PKI_EXT::~PKI_EXT()
{
}


char * PKI_EXT::newpki_strip_spaces(char *name)
{
	char *p, *q;
	/* Skip over leading spaces */
	p = name;
	while(*p && isspace((unsigned char)*p)) p++;
	if(!*p) return NULL;
	q = p + strlen(p) - 1;
	while((q != p) && isspace((unsigned char)*q)) q--;
	if(p != q) q[1] = 0;
	if(!*p) return NULL;
	return p;
}


STACK_OF(CONF_VALUE) * PKI_EXT::newpki_X509V3_parse_list(const char *line)
{
	char *p, *q, *ptr, c;
	char *ntmp, *vtmp;
	STACK_OF(CONF_VALUE) *values = NULL;
	char *linebuf;
	int state;
	/* We are going to modify the line so copy it first */
	linebuf = BUF_strdup(line);
	state = 1;
	ntmp = NULL;
	/* Go through all characters */
	for(p = linebuf, q = linebuf; (c = *p) && (c!='\r') && (c!='\n'); p++) {

		switch(state) {
			case 1:
			if(c == ':') {
				state = 2;
				*p = 0;
				ntmp = newpki_strip_spaces(q);
				if(!ntmp) {
					X509V3err(X509V3_F_X509V3_PARSE_LIST, X509V3_R_INVALID_NULL_NAME);
					goto err;
				}
				q = p + 1;
			} else if(c == ',') {
				*p = 0;
				ntmp = newpki_strip_spaces(q);
				q = p + 1;
				if(!ntmp) {
					X509V3err(X509V3_F_X509V3_PARSE_LIST, X509V3_R_INVALID_NULL_NAME);
					goto err;
				}
				X509V3_add_value(ntmp, NULL, &values);
			}
			break ;

			case 2:
			if(c == ',')
			{
				if(*(p+1) != ',')
				{
					state = 1;
					*p = 0;
					vtmp = newpki_strip_spaces(q);

					if(!vtmp) {
						X509V3err(X509V3_F_X509V3_PARSE_LIST, X509V3_R_INVALID_NULL_VALUE);
						goto err;
					}
					X509V3_add_value(ntmp, vtmp, &values);
					ntmp = NULL;
					q = p + 1;
				}
				else
				{
					for(ptr = p+1; *ptr; ptr++)
					{
						*ptr = *(ptr+1);
					}
				}
			}
		}
	}

	if(state == 2) {
		vtmp = newpki_strip_spaces(q);
		if(!vtmp) {
			X509V3err(X509V3_F_X509V3_PARSE_LIST, X509V3_R_INVALID_NULL_VALUE);
			goto err;
		}
		X509V3_add_value(ntmp, vtmp, &values);
	} else {
		ntmp = newpki_strip_spaces(q);
		if(!ntmp) {
			X509V3err(X509V3_F_X509V3_PARSE_LIST, X509V3_R_INVALID_NULL_NAME);
			goto err;
		}
		X509V3_add_value(ntmp, NULL, &values);
	}
OPENSSL_free(linebuf);
return values;

err:
OPENSSL_free(linebuf);
sk_CONF_VALUE_pop_free(values, X509V3_conf_free);
return NULL;

}


X509_EXTENSION * PKI_EXT::newpki_do_ext_i2d(X509V3_EXT_METHOD *method, int ext_nid,  int crit, void *ext_struc)
{
	unsigned char *ext_der;
	int ext_len;
	ASN1_OCTET_STRING *ext_oct;
	X509_EXTENSION *ext;
	/* Convert internal representation to DER */
	if (method->it)
	{
		ext_der = NULL;
		ext_len = ASN1_item_i2d((ASN1_VALUE *)ext_struc, &ext_der, ASN1_ITEM_ptr(method->it));
		if (ext_len < 0) goto merr;
	}
	 else
	{
		unsigned char *p;
		ext_len = method->i2d(ext_struc, NULL);
		if(!(ext_der = (unsigned char*)OPENSSL_malloc(ext_len)))
			goto merr;
		p = ext_der;
		method->i2d(ext_struc, &p);
	}
	if (!(ext_oct = M_ASN1_OCTET_STRING_new()))
		goto merr;
	ext_oct->data = ext_der;
	ext_oct->length = ext_len;

	ext = X509_EXTENSION_create_by_NID(NULL, ext_nid, crit, ext_oct);
	if (!ext) goto merr;
	M_ASN1_OCTET_STRING_free(ext_oct);

	return ext;

	merr:
	X509V3err(X509V3_F_DO_EXT_I2D,ERR_R_MALLOC_FAILURE);
	return NULL;
}



X509_EXTENSION * PKI_EXT::newpki_do_ext(X509V3_CTX *ctx, int ext_nid, int crit, char *value)
{
	X509V3_EXT_METHOD *method;
	X509_EXTENSION *ext;
	STACK_OF(CONF_VALUE) *nval;
	void *ext_struc;
	if (ext_nid == NID_undef)
	{
		X509V3err(X509V3_F_DO_EXT_CONF,X509V3_R_UNKNOWN_EXTENSION_NAME);
		return NULL;
	}
	if (!(method = X509V3_EXT_get_nid(ext_nid)))
	{
		X509V3err(X509V3_F_DO_EXT_CONF,X509V3_R_UNKNOWN_EXTENSION);
		return NULL;
	}
	/* Now get internal extension representation based on type */
	if (method->v2i)
	{
		nval = newpki_X509V3_parse_list(value);
		if(!nval)
		{
			X509V3err(X509V3_F_X509V3_EXT_CONF,X509V3_R_INVALID_EXTENSION_STRING);
			ERR_add_error_data(4, "name=", OBJ_nid2sn(ext_nid), ",section=", value);
			return NULL;
		}
		ext_struc = method->v2i(method, ctx, nval);
		sk_CONF_VALUE_pop_free(nval, X509V3_conf_free);
		if(!ext_struc)
			return NULL;
	}
	else if(method->s2i)
	{
		if(!(ext_struc = method->s2i(method, ctx, value)))
			return NULL;
	}
	else if(method->r2i)
	{
		if(!ctx->db)
		{
			X509V3err(X509V3_F_X509V3_EXT_CONF,X509V3_R_NO_CONFIG_DATABASE);
			return NULL;
		}
		if(!(ext_struc = method->r2i(method, ctx, value)))
			return NULL;
	}
	else
	{
		X509V3err(X509V3_F_X509V3_EXT_CONF,X509V3_R_EXTENSION_SETTING_NOT_SUPPORTED);
		ERR_add_error_data(2, "name=", OBJ_nid2sn(ext_nid));
		return NULL;
	}

	ext = newpki_do_ext_i2d(method, ext_nid, crit, ext_struc);
	if(method->it)
		ASN1_item_free((ASN1_VALUE *)ext_struc, ASN1_ITEM_ptr(method->it));
	else
		method->ext_free(ext_struc);
	return ext;
}


int PKI_EXT::newpki_v3_check_critical(char **value)
{
	char *p = *value;
	if ((strlen(p) < 9) || strncmp(p, "critical,", 9)) return 0;
	p+=9;
	while(isspace((unsigned char)*p)) p++;
	*value = p;
	return 1;
}

int PKI_EXT::newpki_v3_check_generic(char **value)
{
	char *p = *value;
	if ((strlen(p) < 4) || strncmp(p, "DER:,", 4)) return 0;
	p+=4;
	while (isspace((unsigned char)*p)) p++;
	*value = p;
	return 1;
}

X509_EXTENSION * PKI_EXT::newpki_v3_generic_extension(const char *ext, char *value, int crit, int type)
{
	unsigned char *ext_der=NULL;
	long ext_len;
	ASN1_OBJECT *obj=NULL;
	ASN1_OCTET_STRING *oct=NULL;
	X509_EXTENSION *extension=NULL;
	if (!(obj = OBJ_txt2obj(ext, 0)))
		{
		goto err;
		}

	if (!(ext_der = string_to_hex(value, &ext_len)))
	{
		goto err;
		}

	if (!(oct = M_ASN1_OCTET_STRING_new()))
	{
		NEWPKIerr(CRYPTO_ERROR_TXT, ERROR_MALLOC);
		goto err;
	}

	oct->data = ext_der;
	oct->length = ext_len;
	ext_der = NULL;

	extension = X509_EXTENSION_create_by_OBJ(NULL, obj, crit, oct);

	err:
	ASN1_OBJECT_free(obj);
	M_ASN1_OCTET_STRING_free(oct);
	if(ext_der) OPENSSL_free(ext_der);
	return extension;
}


X509_EXTENSION * PKI_EXT::GetExtensionValue(X509V3_CTX *ctx, const char * Name, const char * Value)
{
	int crit;
	int ext_type;
	X509_EXTENSION *ret;
	crit = newpki_v3_check_critical((char**)&Value);

	if ( (ext_type = newpki_v3_check_generic((char**)&Value)) ) 
	{
		return newpki_v3_generic_extension((char*)Name, (char*)Value, crit, ext_type);
	}

	ret = newpki_do_ext(ctx, OBJ_sn2nid((char*)Name), crit, (char*)Value);
	if (!ret)
	{
		NEWPKIerr(CRYPTO_ERROR_TXT, ERROR_ABORT);
		return NULL;
	}
	return ret;
}

bool PKI_EXT::Add_CertExtensions(const HashTable_String * Ext, X509V3_CTX *ctx, X509 *cert)
{
	X509_EXTENSION *ext;
	long i;
	const char * name;
	const char * value;

	for(i=0;i<Ext->EntriesCount();i++)
	{
		name=Ext->GetName(i);
		value=Ext->Get(i);
		if(!name || !value) continue;
		name = FormatObject(name);

		if(!VerifyExtensionValue(name, value))
		{
			NEWPKIerr(CRYPTO_ERROR_TXT, ERROR_BAD_EXT_SYNTAX);
			ERR_add_error_data(1, name);
			return false;
		}

		if(!(ext = GetExtensionValue(ctx, name, value)))
		{
			return false;
		}
		
		if(cert)
		{
			if(!X509_add_ext(cert, ext, -1))
			{
				X509_EXTENSION_free(ext);
				return false;
			}
		}
		X509_EXTENSION_free(ext);
	}
		
	return true;
}

bool PKI_EXT::Add_CrlExtensions(const HashTable_String * Ext, X509V3_CTX *ctx, X509_CRL *crl)
{
	X509_EXTENSION *ext;
	long i;
	const char * name;
	const char * value;

	for(i=0;i<Ext->EntriesCount();i++)
	{
		name=Ext->GetName(i);
		value=Ext->Get(i);
		if(!name || !value) continue;
		name = FormatObject(name);

		if(!VerifyExtensionValue(name, value))
		{
			NEWPKIerr(CRYPTO_ERROR_TXT, ERROR_BAD_EXT_SYNTAX);
			ERR_add_error_data(1, name);
			return false;
		}


		if(!(ext = GetExtensionValue(ctx, name, value)))
		{
			return false;
		}

		if(crl)
		{
			if(!X509_CRL_add_ext(crl, ext, -1))
			{
				X509_EXTENSION_free(ext);
				return false;
			}
		}
		X509_EXTENSION_free(ext);
	}
	
	return true;
}

bool PKI_EXT::VerifyExtensionValue(const char *FieldName, const char *FieldValue)
{
	X509V3_CTX ctx;
	X509V3_set_ctx_test(&ctx);
	X509_EXTENSION *ext;

	if(!IsValidObject(FieldName))
	{
		return false;
	}
	FieldName = FormatObject(FieldName);

	if (!(ext = GetExtensionValue(&ctx, (char*)FieldName, (char*)FieldValue)))
	{
		ERR_clear_error();
		return false;
	}
	X509_EXTENSION_free(ext);

	ERR_clear_error();
	return true;
}


bool PKI_EXT::Load(const STACK_OF(X509_EXTENSION) * Exts)
{
	int i;
	int nid;
	ASN1_OBJECT *obj;
	X509_EXTENSION * ex;
	char * extvalue;
	char * extname;
	BIO * membio;


	m_Exts.Clear();
	m_Exts.AllowDuplicateNames();
	
	for (i=0; i<sk_X509_EXTENSION_num(Exts); i++)
	{
		ex=sk_X509_EXTENSION_value(Exts, i);

		obj=X509_EXTENSION_get_object(ex);
		if(!obj) continue;

		nid = OBJ_obj2nid(obj);
		if(nid == NID_undef) continue;

		extname = (char*)OBJ_nid2sn(nid);
		if(!extname) continue;

		membio = BIO_new(BIO_s_mem());
		if(!membio) continue;

		if(X509_EXTENSION_get_critical(ex))
		{
			BIO_printf(membio, "%s", "critical, ");
		}
					
		if(!X509V3_EXT_print(membio, ex, X509V3_EXT_MULTILINE, 0))
		{
			M_ASN1_OCTET_STRING_print(membio,ex->value);
		}

		extvalue = (char *)malloc(BIO_number_written(membio) + 1);
		if(!extvalue)
		{
			BIO_free_all(membio);
			continue;
		}

		BIO_read(membio, extvalue, BIO_number_written(membio));
		extvalue[BIO_number_written(membio)]=0;
					
		BIO_free_all(membio);

		//TODO trouver le moyen d'inserer l'info critical
		m_Exts.Add(extname, extvalue);
		free(extvalue);
	}

	return true;
}

const HashTable_String & PKI_EXT::GetExts() const
{
	return m_Exts;
}

void PKI_EXT::Clear()
{
	m_Exts.Clear();
}

bool PKI_EXT::operator=( const PKI_EXT &other )
{
	m_Exts = other.m_Exts;

	return true;
}
