/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/


// PkiClient.cpp: implementation of the PkiClient class.
//
//////////////////////////////////////////////////////////////////////

#include "PkiClient.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
THREAD_ERROR::THREAD_ERROR()
{
}

THREAD_ERROR::THREAD_ERROR(const THREAD_ERROR & other)
{
	*this = other;
}

THREAD_ERROR::~THREAD_ERROR()
{
}

bool THREAD_ERROR::operator=(const THREAD_ERROR & other)
{
	m_line = other.m_line;
	m_file = other.m_file;
	m_lib = other.m_lib;
	m_function = other.m_function;
	m_code = other.m_code;
	m_datas = other.m_datas;

	return true;
}

int THREAD_ERROR::get_Line()
{
	return m_line;
}

void THREAD_ERROR::set_Line(int Line)
{
	m_line = Line;
}

int THREAD_ERROR::get_Lib()
{
	return m_lib;
}

void THREAD_ERROR::set_Lib(int Lib)
{
	m_lib = Lib;
}

int THREAD_ERROR::get_Function()
{
	return m_function;
}

void THREAD_ERROR::set_Function(int Function)
{
	m_function = Function;
}

int THREAD_ERROR::get_Code()
{
	return m_code;
}

void THREAD_ERROR::set_Code(int Code)
{
	m_code = Code;
}

const mString & THREAD_ERROR::get_File()
{
	return m_file;
}

void THREAD_ERROR::set_File(const mString & File)
{
	m_file = File;
}

const mString & THREAD_ERROR::get_Datas()
{
	return m_datas;
}

void THREAD_ERROR::set_Datas(const mString & Datas)
{
	m_datas = Datas;
}

bool THREAD_ERROR::FromThreadErr()
{
	const char *file,*data=NULL;
	unsigned long lastError;
	int flags;

	lastError = ERR_get_error_line_data(&file,&m_line,&data,&flags);
	if(!lastError)
		return false;

	m_lib = ERR_GET_LIB(lastError);
	m_code = ERR_GET_REASON(lastError);
	m_function = ERR_GET_FUNC(lastError);
	if(data && (flags & ERR_TXT_STRING) ) m_datas = data;
	if(file) m_file = file;

	return true;
}















vector<SOCKET> PkiClient::ListAllConnections;
CriticalSection PkiClient::ListAllConnectionsLock;
CriticalSection PkiClient::DatasCounterLock;
unsigned long PkiClient::m_RcvDatas=0;
unsigned long PkiClient::m_SndDatas=0;


PkiClient::PkiClient(PROC_WAIT_CALLBACK * WaitFunction)
{
	ssl_bio = NULL;
	ctx = NULL;
	ssl = NULL;
	m_laSocket = 0;
	m_entity_cert = NULL;
	m_ConnParam.m_MsAuthCert = NULL;
	str_Error = NULL;
	m_WaitFunction = WaitFunction;
	m_ReadTimeOut = 0;

}

PkiClient::~PkiClient()
{
	Disconnect();
#if defined(_WIN32) && !defined(NO_BIO_MS_SSL)
	if(m_ConnParam.m_MsAuthCert) CertFreeCertificateContext(m_ConnParam.m_MsAuthCert);;
#endif

	if(str_Error)
	{
		free(str_Error);
		str_Error = NULL;
	}

	ClearErrors();
}

void PkiClient::SetReadTimeOut(int ReadTimeOut)
{
	m_ReadTimeOut = ReadTimeOut;
}

bool PkiClient::Connect(char *Server, unsigned int Port, const PKI_CERT & AuthCert, PCCERT_CONTEXT MsAuthCert)
{
	ERR_clear_error();

	strcpy(m_ConnParam.Server, Server);
	m_ConnParam.Port = Port;
#if defined(_WIN32) && !defined(NO_BIO_MS_SSL)
	m_ConnParam.m_MsAuthCert = CertDuplicateCertificateContext(MsAuthCert);;
#else
	m_ConnParam.m_MsAuthCert = NULL;
#endif
	if(AuthCert)
		m_ConnParam.m_AuthCert = AuthCert;
	
	return Reconnect();
}


char * PkiClient::GetError()
{
	size_t i;
	
	ERR_clear_error();

	if(str_Error)
	{
		free(str_Error);
		str_Error = NULL;
	}

	for(i=0; i<errors.size(); i++)
	{
		ERR_put_error(errors[i].get_Lib(), errors[i].get_Function(), errors[i].get_Code(), errors[i].get_File().c_str(), errors[i].get_Line());
		if(errors[i].get_Datas().size())
		{
			ERR_add_error_data(1, errors[i].get_Datas().c_str());
		}
	}

	str_Error = ERR_to_string();

	return str_Error;
}


void PkiClient::ClearErrors()
{
	errors.clear();
	ERR_clear_error();
}

void PkiClient::PackThreadErrors()
{
	THREAD_ERROR error;

	errors.clear();

	while(error.FromThreadErr())
	{
		errors.push_back(error);
	}
	ERR_clear_error();
}


void PkiClient::GetPeerCertificate(PKI_CERT * PeerCert)
{
	if(!ssl) return;
	X509 * cert;

#if defined(_WIN32) && !defined(NO_BIO_MS_SSL)
	if(m_ConnParam.m_MsAuthCert)
	{
		if(!ssl_bio)
			return;
		cert = BIO_ms_ssl_get_peer(ssl_bio);
	}
	else
		cert = SSL_get_peer_certificate(ssl);
#else
	cert = SSL_get_peer_certificate(ssl);
#endif

	if(!cert) return;
	PeerCert->SetCert(cert);
	X509_free(cert);
}

void PkiClient::GetEntityCert(PKI_CERT * entity_cert)
{
	if(!m_entity_cert) return;
	entity_cert->SetCert(m_entity_cert);
}

char * PkiClient::GetRemoteServer()
{
	return m_ConnParam.Server;
}

unsigned int PkiClient::GetRemotePort()
{
	return m_ConnParam.Port;
}

bool PkiClient::Reconnect()
{
	return DoNetworkConnection(&m_ConnParam);
}

bool PkiClient::DoNetworkExchange(PKI_CLIENT_PARAM * ClientParam)
{
	ClearErrors();

	if(m_WaitFunction)
	{
		NewpkiThread ThreadId;
		ThreadId.Create(ThreadProcSocket, ClientParam);
		ClientParam->Finished = false;
		ClientParam->me_this = this;


		if(!ThreadId.Start())
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_UNKNOWN);
			PackThreadErrors();
			return false;
		}
		
		while(!ClientParam->Finished)
		{
			m_WaitFunction();
			NewpkiThread::Sleep(10);
		}
		ThreadId.Stop();
		return ClientParam->Success;
	}
	else
	{
		return DoBufferSend(ClientParam);
	}
}


bool PkiClient::DoNetworkConnection(PKI_CONNECTION_PARAM * ConnParam)
{
	ClearErrors();

	if(m_WaitFunction)
	{
		NewpkiThread ThreadId;
		ThreadId.Create(ThreadProcConnection, ConnParam);

		ConnParam->me_this = this;
		ConnParam->Finished = false;

		if(!ThreadId.Start())
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_UNKNOWN);
			PackThreadErrors();
			return false;
		}
		while(!ConnParam->Finished)
		{
			m_WaitFunction();
			NewpkiThread::Sleep(10);
		}
		ThreadId.Stop();
		return ConnParam->Success;
	}
	else
	{
		return DoConnection(ConnParam);
	}
}


void PkiClient::ThreadProcSocket(const NewpkiThread * Thread, void *param)
{
	PKI_CLIENT_PARAM * ClientParam = (PKI_CLIENT_PARAM *)param;
	ClientParam->Success = ClientParam->me_this->DoBufferSend(ClientParam);
	ClientParam->Finished = true;
}

void PkiClient::ThreadProcConnection(const NewpkiThread * Thread, void *param)
{
	PKI_CONNECTION_PARAM * ConnParam = (PKI_CONNECTION_PARAM *)param;
	ConnParam->Success = ConnParam->me_this->DoConnection(ConnParam);
	ConnParam->Finished = true;
}


bool PkiClient::DoBufferSend(PKI_CLIENT_PARAM * ClientParam)
{
	ClearErrors();

	ADMIN_REQUEST * command;
	ERROR_ENTRY * currEntry;
	int i;
	THREAD_ERROR error;
	fd_set rfds;
	struct timeval tv;
	

	command = ADMIN_REQUEST_new();
	if(!command)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_MALLOC);
		PackThreadErrors();
		return false;
	}

	command->body->type = ClientParam->command;
	command->body->d.other = (ASN1_NULL*)ClientParam->a;


	if(!i2d_ADMIN_REQUEST_bio(ssl_bio, command))
	{
		//BODY is freed by calling function

		ERR_clear_error();

		command->body->d.other = NULL;
		ADMIN_REQUEST_free(command);
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_CONNECTION_LOST);
		PackThreadErrors();
		return false;
	}
	//BODY is freed by calling function
	command->body->d.other = NULL;
	ADMIN_REQUEST_free(command);

	//Get response
	if(m_ReadTimeOut)
	{
		FD_ZERO(&rfds);
		FD_SET(m_laSocket, &rfds);
		tv.tv_sec = m_ReadTimeOut;
		tv.tv_usec = 0;
		
		if(select(m_laSocket + 1, &rfds, NULL, NULL, &tv) <= 0)
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_CONNECTION_LOST);
			PackThreadErrors();
			return false;
		}
	}
	
	if( !(ClientParam->response = d2i_ADMIN_RESPONSE_bio(ssl_bio, NULL)) )
	{
#ifdef _DEBUG
		ERR_print_errors_fp(stdout);
#endif
		ERR_clear_error();
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_CONNECTION_LOST);
		PackThreadErrors();
		return false;
	}

	//Do we have the response signer cert yet ?
	if(!ClientParam->response->signing_cert)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_RESPONSE_SIGNATURE);
		ADMIN_RESPONSE_free(ClientParam->response);
		ClientParam->response = NULL;
		PackThreadErrors();
		return false;
	}
	if(!m_entity_cert)
	{
		m_entity_cert = X509_dup(ClientParam->response->signing_cert);
	}
	else
	{
		// Is the signing cert the same ?
		if(X509_cmp(ClientParam->response->signing_cert, m_entity_cert) != 0)
		{
			//NO: that's an error, the cert can't change during a session !
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_RESPONSE_SIGNATURE);
			ADMIN_RESPONSE_free(ClientParam->response);
			ClientParam->response = NULL;
			PackThreadErrors();
			return false;
		}
	}

	EVP_PKEY * pubKey = X509_get_pubkey(m_entity_cert);
	if(!pubKey)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		ADMIN_RESPONSE_free(ClientParam->response);
		ClientParam->response = NULL;
		PackThreadErrors();
		return false;
	}

	//Let's verify the response
	if(ASN1_verify((int (*)(void))i2d_ADMIN_RESPONSE_BODY, ClientParam->response->sig_algo, ClientParam->response->signature, (char*)ClientParam->response->body, pubKey) <= 0 )
	{
		EVP_PKEY_free(pubKey);
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		ADMIN_RESPONSE_free(ClientParam->response);
		ClientParam->response = NULL;
		PackThreadErrors();
		return false;
	}
	EVP_PKEY_free(pubKey);



	
	if(ClientParam->response->body->type == ADMIN_RESP_TYPE_ERRORS)
	{

		for(i=0 ; i < SKM_sk_num(ERROR_ENTRY, ClientParam->response->body->d.errors); i++)
		{
			currEntry = SKM_sk_value(ERROR_ENTRY, ClientParam->response->body->d.errors, i);
			if(!currEntry) continue;

			error.set_Code(ASN1_INTEGER_GET(currEntry->code));
			error.set_Lib(ASN1_INTEGER_GET(currEntry->lib));
			error.set_Function(ASN1_INTEGER_GET(currEntry->function));
			error.set_Line(ASN1_INTEGER_GET(currEntry->line));
			error.set_File(ASN1_STRING_GET(currEntry->file));
			error.set_Datas(ASN1_STRING_GET(currEntry->data));
			errors.push_back(error);
		}
		ADMIN_RESPONSE_free(ClientParam->response);
		ClientParam->response = NULL;
		return false;
	}

	return true;
}

bool PkiClient::DoConnection(PKI_CONNECTION_PARAM * ConnParam)
{
	ClearErrors();
	Disconnect();

	struct linger ling;
	sockaddr_in sockin;
	struct hostent * hostinf;
	char ip[30];
	int ret;

	hostinf = gethostbyname(ConnParam->Server);
	if(!hostinf || !hostinf->h_length)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_CONNECT_FAILED);
		PackThreadErrors();
		return false;
	}

	sprintf(ip, "%d.%d.%d.%d", (unsigned char)hostinf->h_addr_list[0][0], (unsigned char)hostinf->h_addr_list[0][1], (unsigned char)hostinf->h_addr_list[0][2], (unsigned char)hostinf->h_addr_list[0][3]);
	
	memset(&sockin, 0, sizeof(sockin));
	sockin.sin_family = AF_INET;
	sockin.sin_port = htons (ConnParam->Port);
	sockin.sin_addr.s_addr=inet_addr(ip);

	m_laSocket = socket(PF_INET, SOCK_STREAM, 0);
	if(m_laSocket == SOCKET_ERROR)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_SOCKET);
		PackThreadErrors();
		return false;
	}

	ling.l_onoff=1;
	ling.l_linger=0;
	if(setsockopt(m_laSocket, SOL_SOCKET, SO_LINGER, (const char*) &ling, sizeof(ling)) < 0)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_SOCKET);
		PackThreadErrors();
		return false;
	}

	if(connect(m_laSocket,(sockaddr *)&sockin, sizeof(sockin)) == SOCKET_ERROR)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_CONNECT_FAILED);
		PackThreadErrors();
		Disconnect();
		return false;
	}

#if defined(_WIN32) && !defined(NO_BIO_MS_SSL)
	BIO * sbio;
	//We use MS SSL API
	if(ConnParam->m_MsAuthCert)
	{
		sbio = BIO_new_socket(m_laSocket, BIO_CLOSE);
		if(!sbio)
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_MALLOC);
			PackThreadErrors();
			Disconnect();
			return false;
		}
		ssl_bio = BIO_new_ms_ssl(sbio, (PCERT_CONTEXT)ConnParam->m_MsAuthCert);
		if(!ssl_bio)
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_MALLOC);
			PackThreadErrors();
			BIO_free_all(sbio);
			Disconnect();
			return false;
		}
		return true;
	}
#endif



	//We use OpenSSL SSL API
	SSL_METHOD * meth;
	
	ERR_clear_error();
/*
	if(ConnParam->m_AuthCert)
		meth = SSLv3_client_method();
	else
		meth = SSLv2_client_method();
*/
	meth = SSLv23_client_method();
	if(!meth)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		PackThreadErrors();
		Disconnect();
		return false;
	}
	ctx = SSL_CTX_new (meth);
	if(!ctx)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_MALLOC);
		PackThreadErrors();
		Disconnect();
		return false;
	}

	if(ConnParam->m_AuthCert)
	{
		if (SSL_CTX_use_certificate(ctx, ConnParam->m_AuthCert.GetX509()) <= 0)
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
			PackThreadErrors();
			Disconnect();
			return false;
		}
		if (SSL_CTX_use_PrivateKey(ctx, (EVP_PKEY*)ConnParam->m_AuthCert.GetPrivateKey().GetRsaKey()) <= 0) 
		{
			NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
			PackThreadErrors();
			Disconnect();
			return false;
		}
	}


	SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE, verify_callback);


	if( !(ssl=SSL_new(ctx)) )
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_MALLOC);
		PackThreadErrors();
		Disconnect();
		return false;
	}

	SSL_set_mode(ssl, SSL_MODE_AUTO_RETRY);

	SSL_set_fd(ssl,m_laSocket);

	ssl_bio = BIO_new(BIO_f_ssl());
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_MALLOC);
		PackThreadErrors();
		Disconnect();
		return false;
	}
	if(BIO_set_ssl(ssl_bio, ssl, BIO_CLOSE) <= 0)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		PackThreadErrors();
		Disconnect();
		return false;
	}

	if(BIO_set_ssl_mode(ssl_bio, 1) <= 0)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		PackThreadErrors();
		Disconnect();
		return false;
	}


	ERR_clear_error();
	if( (ret = SSL_connect(ssl)) <= 0)
	{
		switch(SSL_get_error(ssl, ret))
		{
			case SSL_ERROR_SSL:
				NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
				break;
			case SSL_ERROR_SYSCALL:
				NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
				ERR_add_error_data(1, strerror(errno));
				break;
			default:
				NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
				break;
		}

		PackThreadErrors();
		Disconnect();
		return false;
	}

	ListAllConnectionsLock.Lock();
	ListAllConnections.push_back(m_laSocket);
	ListAllConnectionsLock.Unlock();

	return true;
}


int PkiClient::verify_callback(int ok, X509_STORE_CTX *ctx)
{
	return 1;
}


bool PkiClient::UserLogin(char * entity, char *username, char *password, int & UserType)
{
	ClearErrors();
	if(!ssl_bio)
	{		
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}


	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_LOGIN, ADMIN_REQ_LOGIN);

	if(entity && *entity) NET_PARAM_SET_STR(entity, entity);
	if(username && *username) NET_PARAM_SET_STR(username, username);
	if(password && *password) NET_PARAM_SET_STR(password, password);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ADMIN_REQ_LOGIN);
		return false;
	}

	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_USER_TYPE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ADMIN_REQ_LOGIN);
		PackThreadErrors();
		return false;
	}
	
	UserType = ASN1_INTEGER_GET(NET_PARAM_RESPONSE()->body->d.usertype);

	FREE_NET_PARAM(ADMIN_REQ_LOGIN);

	return true;
}

bool PkiClient::EnumEntities(STACK_OF(ENTITY_ENTRY_INFO) ** entities)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ENUM_ENTITIES, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENUM_ENTITIES)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	*entities = NET_PARAM_RESPONSE()->body->d.entities;
	//Calling function will free stack
	NET_PARAM_RESPONSE()->body->d.entities=NULL;
	FREE_NET_PARAM(ASN1_NULL);

	return true;
}



bool PkiClient::CreateEntity(ENTITY_CREATION_REQ *cre_entity, ENTITY_CREATION_RESP ** creation_resp)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_CREATE_ENTITY, cre_entity);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CREATE_ENTITY)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	*creation_resp = NET_PARAM_RESPONSE()->body->d.cre_entity;
	//Calling function will free stack
	NET_PARAM_RESPONSE()->body->d.cre_entity=NULL;
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::DeleteEntity(const char *Name)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}


	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_DELETE_ENTITY, ASN1_UTF8STRING);

	ASN1_STRING_SET(&ttvalue, Name);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_UTF8STRING);
		return false;
	}

	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_UTF8STRING);
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM(ASN1_UTF8STRING);

	return true;
}

bool PkiClient::EnumLOG(ADMIN_REQ_ENUM_LOGS * Filters, STACK_OF(LOG_ENTRY) ** Logs)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}


	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_ENUM_LOGS, Filters);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}

	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENUM_LOGS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	*Logs = NET_PARAM_RESPONSE()->body->d.logs;
	//Calling function will free stack
	NET_PARAM_RESPONSE()->body->d.logs=NULL;
	FREE_NET_PARAM_ANY();

	return true;
}

bool PkiClient::GetLogsCount(ADMIN_REQ_ENUM_LOGS * Filters, unsigned long & LogsCount)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_LOGS_COUNT, Filters);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}

	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_LOGS_COUNT)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	LogsCount = ASN1_INTEGER_GET(NET_PARAM_RESPONSE()->body->d.status);
	FREE_NET_PARAM_ANY();

	return true;
}



bool PkiClient::EnumUsers(STACK_OF(USER_ENTRY) ** users)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ENUM_USERS, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENUM_USERS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	*users = NET_PARAM_RESPONSE()->body->d.users;
	//Calling function will free stack
	NET_PARAM_RESPONSE()->body->d.users=NULL;
	FREE_NET_PARAM(ASN1_NULL);

	return true;
}

bool PkiClient::CreateUser(USER_INFO *UserInfo)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_CREATE_USER, UserInfo);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::UpdateUser(USER_ENTRY *UserEntry)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_UPDATE_USER, UserEntry);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::ChangeUserPassword(CHANGE_USER_PASSWD *Password)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_CHG_USER_PASSWD, Password);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::ChangePassword(CHANGE_PASSWD *Password)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_CHG_PASSWD, Password);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM_ANY();
	return true;
}


bool PkiClient::SignEntity(ENTITY_SIGNATURE_REQ *sign_entity, ENTITY_SIGNATURE_RESP **resp)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_SIGN_ENTITY, sign_entity);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_SIGN_ENTITY)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	*resp = NET_PARAM_RESPONSE()->body->d.sign_entity;
	//Calling function will free response
	NET_PARAM_RESPONSE()->body->d.logs=NULL;
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::EnumUsersCert(STACK_OF(USER_ENTRY_CERT) ** users_cert)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ENUM_USERS, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENUM_USERS_CERT)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	*users_cert = NET_PARAM_RESPONSE()->body->d.users_cert;
	//Calling function will free stack
	NET_PARAM_RESPONSE()->body->d.users_cert=NULL;
	FREE_NET_PARAM(ASN1_NULL);

	return true;
}


bool PkiClient::CreatePkiUser(X509_REQ *req, int PkiAdmin, STACK_OF(X509) ** certs)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_CREATE_PKI_USER, CREATE_PKI_USER);
	
	if(!X509_REQ_cpy(&ttvalue->csr, req))
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		PackThreadErrors();
		return false;
	}
	ASN1_INTEGER_SET(&ttvalue->pkiadmin, PkiAdmin);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(CREATE_PKI_USER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CREATE_PKI_USER)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(CREATE_PKI_USER);
		PackThreadErrors();
		return false;
	}

	*certs = NET_PARAM_RESPONSE()->body->d.create_pki_user;
	//Calling function will free stack
	NET_PARAM_RESPONSE()->body->d.users_cert=NULL;
	FREE_NET_PARAM(CREATE_PKI_USER);

	return true;
}

bool PkiClient::RevokeUserCert(unsigned long serial)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_REVOKE_USER, ASN1_INTEGER);
	ASN1_INTEGER_SET(&ttvalue, serial);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_INTEGER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_INTEGER);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(ASN1_INTEGER);

	return true;
}

bool PkiClient::RevokeEntityCert(unsigned long serial)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_REVOKE_ENTITY, ASN1_INTEGER);
	ASN1_INTEGER_SET(&ttvalue, serial);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_INTEGER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_INTEGER);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(ASN1_INTEGER);

	return true;
}

bool PkiClient::PushConfiguration(EXPORTED_PKI_CONF *conf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_PUSH_CONFIG, conf);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::SetOfflineState(bool offline)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SET_OFFLINE, ASN1_INTEGER);

	ASN1_INTEGER_SET(&ttvalue, (offline?1:0));

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_INTEGER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_INTEGER);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(ASN1_INTEGER);

	return true;
}

bool PkiClient::GetOfflineState(bool & offline)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_OFFLINE, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_STATUS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	offline = (ASN1_INTEGER_GET(NET_PARAM_RESPONSE()->body->d.status) > 0);

	FREE_NET_PARAM(ASN1_NULL);

	return true;
}

bool PkiClient::GetMyACL(ASN1_BIT_STRING **acl)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_MY_ACL, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_MY_ACL)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	*acl = NET_PARAM_RESPONSE()->body->d.my_acl;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.my_acl=NULL;
	FREE_NET_PARAM(ASN1_NULL);

	return true;
}

bool PkiClient::GetConfiguration(EXPORTED_PKI_CONF **conf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_CONFIG, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CONF)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	*conf = NET_PARAM_RESPONSE()->body->d.conf;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.conf=NULL;

	FREE_NET_PARAM(ASN1_NULL);

	return true;
}

bool PkiClient::InitEntity(ENTITY_INIT_REQ *init_entity)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_INIT_ENTITY, init_entity);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	FREE_NET_PARAM_ANY();	
	return true;
}

bool PkiClient::GetRepositoriesList(REPS_INFO **repositories)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_REPOSITORIES, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_GET_REPOSITORIES)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	*repositories = NET_PARAM_RESPONSE()->body->d.repositories;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.repositories=NULL;
	
	FREE_NET_PARAM(ASN1_NULL);
	
	return true;
}

bool PkiClient::SetRepositoriesList(REPS_INFO *repositories)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_SET_REPOSITORIES, repositories);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	FREE_NET_PARAM_ANY();	
	return true;
}

bool PkiClient::GetLogsType(STACK_OF(ASN1_INTEGER) ** LogsType)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_LOGS_TYPE, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_LOGS_TYPE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	*LogsType = NET_PARAM_RESPONSE()->body->d.logs_type;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;
	
	FREE_NET_PARAM(ASN1_NULL);
	
	return true;
}


bool PkiClient::GetEntityConf(X509 * entity_cert, ENTITY_CONF_BODY ** conf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_ENTITY_CONF, entity_cert);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENTITY_CONF)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*conf = NET_PARAM_RESPONSE()->body->d.entity_conf;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::ResourceLock(char *ResourceName)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}


	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_LOCK_RESOURCE, ASN1_UTF8STRING);

	ASN1_STRING_SET(&ttvalue, ResourceName);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_UTF8STRING);
		return false;
	}

	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_UTF8STRING);
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM(ASN1_UTF8STRING);

	return true;
}

bool PkiClient::ResourceUnlock(char *ResourceName)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}


	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_UNLOCK_RESOURCE, ASN1_UTF8STRING);

	ASN1_STRING_SET(&ttvalue, ResourceName);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_UTF8STRING);
		return false;
	}

	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_UTF8STRING);
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM(ASN1_UTF8STRING);

	return true;
}

bool PkiClient::GetEntityAcl(X509 *entity_cert, X509_ACL **acl)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_ENTITY_ACL, entity_cert);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENTITY_ACL)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*acl = NET_PARAM_RESPONSE()->body->d.acls_conf;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GetEntityMailConf(X509 *entity_cert, EMAIL_CONF **mail_conf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_ENTITY_MAIL_CONF, entity_cert);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_MAIL_CONF)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*mail_conf = NET_PARAM_RESPONSE()->body->d.mail_conf;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::SetEntityMailConf(X509 *entity_cert, EMAIL_CONF *mail_conf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SET_ENTITY_MAIL_CONF, SET_ENTITY_MAIL_CONF);

	X509_cpy(&ttvalue->entity_cert, entity_cert);
	EMAIL_CONF_cpy(&ttvalue->mail_conf, mail_conf);

	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(SET_ENTITY_MAIL_CONF);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(SET_ENTITY_MAIL_CONF);
		PackThreadErrors();
		return false;
	}
		
	FREE_NET_PARAM(SET_ENTITY_MAIL_CONF);
	
	return true;
}

bool PkiClient::SetEntityAcl(X509 *entity_cert, STACK_OF(ACL_ENTRY) * acl_entries)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SET_ENTITY_ACL, SET_ENTITY_ACL);

	X509_cpy(&ttvalue->entity_cert, entity_cert);
	sk_free(ttvalue->acl_entries);
	ttvalue->acl_entries = acl_entries;

	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		ttvalue->acl_entries = NULL;
		FREE_NET_PARAM(SET_ENTITY_ACL);
		return false;
	}
	ttvalue->acl_entries = NULL;


	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(SET_ENTITY_ACL);
		PackThreadErrors();
		return false;
	}
		
	FREE_NET_PARAM(SET_ENTITY_ACL);
	
	return true;
}

bool PkiClient::SetEntityConf(X509 *entity_cert, ENTITY_CONF_BODY *conf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SET_ENTITY_CONF, SET_ENTITY_CONF);

	X509_cpy(&ttvalue->entity_cert, entity_cert);
	ENTITY_CONF_BODY_cpy(&ttvalue->conf, conf);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(SET_ENTITY_CONF);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(SET_ENTITY_CONF);
		PackThreadErrors();
		return false;
	}
		
	FREE_NET_PARAM(SET_ENTITY_CONF);
	
	return true;
}

bool PkiClient::AdminSendMail(MAIL_INFO *mail)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_SEND_ADMIN_MAIL, mail);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
		
	FREE_NET_PARAM_ANY();
	return true;
}


bool PkiClient::GetEntityAudits(X509 *entity_cert, ENTITY_AUDIT **audits)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_ENTITY_AUDITS, entity_cert);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_AUDITS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*audits = NET_PARAM_RESPONSE()->body->d.audits;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.audits=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::SetEntityAudits(X509 *entity_cert, ENTITY_AUDIT * audits)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SET_ENTITY_AUDITS, SET_ENTITY_AUDITS);

	if(entity_cert)
		X509_cpy(&ttvalue->entity_cert, entity_cert);
	ENTITY_AUDIT_cpy(&ttvalue->audits, audits);

	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(SET_ENTITY_AUDITS);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(SET_ENTITY_AUDITS);
		PackThreadErrors();
		return false;
	}
		
	FREE_NET_PARAM(SET_ENTITY_AUDITS);
	
	return true;
}


bool PkiClient::GetEntityLogsType(int EntityType, STACK_OF(ASN1_INTEGER) ** LogsType)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_ENTITY_LOGS_TYPE, ASN1_INTEGER);

	ASN1_INTEGER_SET(&ttvalue, EntityType);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_INTEGER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_LOGS_TYPE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_INTEGER);
		PackThreadErrors();
		return false;
	}
	
	*LogsType = NET_PARAM_RESPONSE()->body->d.logs_type;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;
	
	FREE_NET_PARAM(ASN1_INTEGER);
	
	return true;
}

bool PkiClient::SetEntitiesLinks(ENTITIES_LINKS *links)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_SET_ENTITIES_LINKS, links);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GetEntitiesLinks(ENTITIES_LINKS **links)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_ENTITIES_LINKS, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_ENTITIES_LINKS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	*links = NET_PARAM_RESPONSE()->body->d.entities_links;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.audits=NULL;
	
	FREE_NET_PARAM(ASN1_NULL);
	
	return true;
}

bool PkiClient::GetMyConf(ENTITY_CONF_CRYPTED **myConf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_MY_CONFIG, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_MY_CONF)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	*myConf = NET_PARAM_RESPONSE()->body->d.my_conf;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.my_conf=NULL;
	
	FREE_NET_PARAM(ASN1_NULL);
	
	return true;
}

bool PkiClient::PushWaitingObject(WAITING_NEWPKI_OBJECT *objs)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_PUSH_NEWPKI_OBJECTS, objs);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GetCaStatus(ASN1_INTEGER ** status)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_CA_STATUS, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_STATUS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	*status = NET_PARAM_RESPONSE()->body->d.status;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM(ASN1_NULL);
	
	return true;
}

bool PkiClient::CreateCaRoot(REQ_CREATE_ROOT_CA * ca_info)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_CREATE_ROOT_CA, ca_info);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GetMyRequests(TRANSACTION_IDS * transactionIds, STORED_NEWPKI_REQUESTS ** objects)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_MY_REQS, transactionIds);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_REQS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*objects = NET_PARAM_RESPONSE()->body->d.object_reqs;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GetMyResponses(TRANSACTION_IDS * transactionIds, STORED_NEWPKI_RESPONSES ** objects)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_GET_MY_RESPS, transactionIds);


	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_RESPS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*objects = NET_PARAM_RESPONSE()->body->d.object_resps;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::SendRequest(CRYPTED_NEWPKI_REQUEST * req, ASN1_OCTET_STRING ** transactionId)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_PUSH_REQ, req);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_TRANSACTION_ID)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*transactionId = NET_PARAM_RESPONSE()->body->d.transactionID;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM_ANY();
	return true;
}


bool PkiClient::SendResponse(STORED_NEWPKI_RESPONSE * resp)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_PUSH_RESP, resp);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM_ANY();
	return true;
}

void PkiClient::CancelAllConnections()
{
	ListAllConnectionsLock.Lock();
	SOCKET hSocket;
	unsigned int i;	
	for(i=0; i<ListAllConnections.size(); i++)
	{
		hSocket = ListAllConnections.at(i);
		if(!hSocket) continue;
		closesocket(hSocket);
	}
	ListAllConnections.empty();
	ListAllConnectionsLock.Unlock();

}

void PkiClient::CloseConnection()
{
	unsigned int i;
	if(m_laSocket)
	{
		ListAllConnectionsLock.Lock();
		for(i=0; i<ListAllConnections.size(); i++)
		{
			if(ListAllConnections.at(i) == m_laSocket)
			{
				ListAllConnections.erase(ListAllConnections.begin()+i);
				break;
			}
		}
		closesocket(m_laSocket);
		m_laSocket = 0;
		ListAllConnectionsLock.Unlock();
	}
}

void PkiClient::Disconnect()
{
	CloseConnection();
	if(ssl)
	{
		SSL_shutdown(ssl);
		ssl = NULL;
	}
	if(ctx)
	{
		SSL_CTX_free(ctx);
		ctx = NULL;
	}
	if(m_entity_cert)
	{
		X509_free(m_entity_cert);
		m_entity_cert = NULL;
	}
	if(ssl_bio)
	{
		BIO_free_all(ssl_bio);
		ssl_bio = NULL;
	}
}


bool PkiClient::EnumCERT(long index, long num, CERT_STATE state, STACK_OF(INTERNAL_CA_CERT) ** certs)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ENUM_CERTS, ENUM_OBJECTS);
	
	NET_PARAM_SET_INT(state, state);
	NET_PARAM_SET_INT(index, index);
	NET_PARAM_SET_INT(num, num);


	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ENUM_OBJECTS);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CERTS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ENUM_OBJECTS);
		PackThreadErrors();
		return false;
	}
	
	*certs = NET_PARAM_RESPONSE()->body->d.certs;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM(ENUM_OBJECTS);
	
	return true;
}


bool PkiClient::EnumCrls(long index, long num, STACK_OF(X509_CRL) ** crls)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ENUM_CRLS, ENUM_OBJECTS);

	NET_PARAM_SET_INT(state, 0);
	NET_PARAM_SET_INT(index, index);
	NET_PARAM_SET_INT(num, num);


	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ENUM_OBJECTS);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CRLS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ENUM_OBJECTS);
		PackThreadErrors();
		return false;
	}
	
	*crls = NET_PARAM_RESPONSE()->body->d.crls;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM(ENUM_OBJECTS);
	
	return true;
}


bool PkiClient::SignCSR(X509_REQ *req, int Days, PKCS7 ** p7b)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SIGN_CSR, SIGN_CSR);

	NET_PARAM_SET_INT(days, Days);
	if(!X509_REQ_cpy(&ttvalue->request, req))
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(SIGN_CSR);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_P7B)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(SIGN_CSR);
		PackThreadErrors();
		return false;
	}

	*p7b = NET_PARAM_RESPONSE()->body->d.p7b;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;

	FREE_NET_PARAM(SIGN_CSR);

	return true;
}

bool PkiClient::CreateCaChild(REQ_CREATE_CHILD_CA * ca_info, X509_REQ ** csr)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_CREATE_CHILD_CA, ca_info);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CSR)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	*csr = NET_PARAM_RESPONSE()->body->d.csr;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;

	FREE_NET_PARAM_ANY();
	return true;
}


bool PkiClient::ImportChildCaCert(PKCS7 * certs)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_IMPORT_CHILD_CA_CERT, certs);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::RevokeCert(unsigned long serial)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_REVOKE_CERT, ASN1_INTEGER);
	ASN1_INTEGER_SET(&ttvalue, serial);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_INTEGER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_INTEGER);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(ASN1_INTEGER);

	return true;
}

bool PkiClient::GetGroups(USERS_GROUPS **Groups)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_GROUPS, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_GROUPS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	*Groups = NET_PARAM_RESPONSE()->body->d.groups;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;

	FREE_NET_PARAM(ASN1_NULL);
	
	return true;
}

bool PkiClient::SetGroups(USERS_GROUPS *Groups)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_SET_GROUPS, Groups);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::AddGroup(char *Name)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ADD_GROUP, ASN1_UTF8STRING);
	ASN1_STRING_SET(&ttvalue, Name);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_UTF8STRING);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_UTF8STRING);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(ASN1_UTF8STRING);

	return true;
}

bool PkiClient::DelGroup(unsigned long id)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_DEL_GROUP, ASN1_INTEGER);
	ASN1_INTEGER_SET(&ttvalue, id);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_INTEGER);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_INTEGER);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(ASN1_INTEGER);

	return true;
}

bool PkiClient::RenameGroup(unsigned long id, char *Name)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_REN_GROUP, RENAME_GROUP);
	ASN1_STRING_SET(&ttvalue->name, Name);
	ASN1_INTEGER_SET(&ttvalue->id, id);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(RENAME_GROUP);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(RENAME_GROUP);
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM(RENAME_GROUP);

	return true;
}


void PkiClient::AddSentBytes(unsigned long bytes)
{
	DatasCounterLock.Lock();
	m_SndDatas+=bytes;
	DatasCounterLock.Unlock();
}

void PkiClient::AddRecvBytes(unsigned long bytes)
{
	DatasCounterLock.Lock();
	m_RcvDatas+=bytes;
	DatasCounterLock.Unlock();
}

unsigned long PkiClient::GetRecvBytes()
{
	return m_RcvDatas;
}

unsigned long PkiClient::GetSentBytes()
{
	return m_SndDatas;
}

bool PkiClient::CheckLogsIntegrity()
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_CHECK_LOGS, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	FREE_NET_PARAM(ASN1_NULL);	
	return true;
}

bool PkiClient::ImportProfile(NEWPKI_PROFILE *Profile, unsigned long & ProfileId)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_IMPORT_PROFILE, Profile);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_PROFILE_ID)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	ProfileId = ASN1_INTEGER_GET(NET_PARAM_RESPONSE()->body->d.id);
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GetLocalConf(ENTITY_CONF **EntityConf)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_LOCAL_CONF, ASN1_NULL);


	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_LOCAL_ENTITY_CONF)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}

	*EntityConf = NET_PARAM_RESPONSE()->body->d.local_entity_conf;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.logs_type=NULL;

	FREE_NET_PARAM(ASN1_NULL);
	return true;
}

bool PkiClient::EnumProfiles(long index, long num, STACK_OF(NEWPKI_PROFILE_DATAS) ** Profiles)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_ENUM_PROFILES, ENUM_OBJECTS);
	
	NET_PARAM_SET_INT(state, 0);
	NET_PARAM_SET_INT(index, index);
	NET_PARAM_SET_INT(num, num);


	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ENUM_OBJECTS);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_PROFILES)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ENUM_OBJECTS);
		PackThreadErrors();
		return false;
	}
	
	*Profiles = NET_PARAM_RESPONSE()->body->d.profiles;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM(ENUM_OBJECTS);
	
	return true;
}


bool PkiClient::RequestCertificate(REQUEST_CERT *cert_request, unsigned long & ReqId)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_REQUEST_CERT, cert_request);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_REQ_ID)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	ReqId = ASN1_INTEGER_GET(NET_PARAM_RESPONSE()->body->d.id);
	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::GenerateCRL()
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}
	
	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GENERATE_CRL, ASN1_NULL);
	
	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	
	FREE_NET_PARAM(ASN1_NULL);	
	return true;
}

bool PkiClient::SynchObjects(OBJECTS_LIST_REQUEST *KnownObjects, OBJECTS_LIST_RESPONSE ** RepKnownObjects)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_SYNCH_OBJECTS, KnownObjects);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_KNOWN_OBJECTS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}
	
	*RepKnownObjects = NET_PARAM_RESPONSE()->body->d.known_objects;
	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM_ANY();
	
	return true;
}

bool PkiClient::GetCaCert(PKI_CERT & cert)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_GET_CA_CERT, ASN1_NULL);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_NULL);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_CERT)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	if(!cert.SetCert(NET_PARAM_RESPONSE()->body->d.cert))
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_ABORT);
		FREE_NET_PARAM(ASN1_NULL);
		PackThreadErrors();
		return false;
	}
	FREE_NET_PARAM(ASN1_NULL);

	return true;
}


bool PkiClient::DeletePkcs12(ASN1_INTEGER * CertId)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_STATIC(ADMIN_REQ_TYPE_DELETE_PKCS12, CertId);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM_ANY();
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_NONE)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM_ANY();
		PackThreadErrors();
		return false;
	}

	FREE_NET_PARAM_ANY();
	return true;
}

bool PkiClient::SearchLdap(const char *Filters, STACK_OF(LDAP_RESULT) ** Results)
{
	ClearErrors();
	if(!ssl_bio)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_NO_CONNECTION);
		PackThreadErrors();
		return false;
	}

	INIT_NET_PARAM_TYPE(ADMIN_REQ_TYPE_SEARCH_LDAP, ASN1_UTF8STRING);
	ASN1_STRING_SET(&ttvalue, Filters);

	if(!DoNetworkExchange(&NET_PARAM()))
	{
		FREE_NET_PARAM(ASN1_UTF8STRING);
		return false;
	}
	if(NET_PARAM_RESPONSE()->body->type != ADMIN_RESP_TYPE_LDAP_RESULTS)
	{
		NEWPKIerr(CLIENT_ERROR_TXT, ERROR_BAD_DATAS);
		FREE_NET_PARAM(ASN1_UTF8STRING);
		PackThreadErrors();
		return false;
	}
	
	*Results = NET_PARAM_RESPONSE()->body->d.ldap_results;

	//Calling function will free datas
	NET_PARAM_RESPONSE()->body->d.status=NULL;
	
	FREE_NET_PARAM(ASN1_UTF8STRING);	
	return true;
}
