/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

// ReadersWriter.cpp: implementation of the ReadersWriter class.
//
//////////////////////////////////////////////////////////////////////

#include "ReadersWriter.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ReadersWriter::ReadersWriter()
{
	write_ctr = 0;
	read_ctr = 0;
}

ReadersWriter::~ReadersWriter()
{

}

void ReadersWriter::LockRead() const
{
	// Wait for all writers to be done
	// It gives priority to writer threads
	LockerAcc.Lock();
	((ReadersWriter*)this)->read_ctr++;
	//Am I the first reader ?
	if(read_ctr == 1)
	{
		write_lock.Wait();	//Take ownership
	}

	LockerAcc.Unlock();
}

void ReadersWriter::LockWrite()
{
	LockerAcc.Lock();
	
	/***************************************
	"Modified the 24/08/2003 by Sandson (sandson@ifrance.com)
	***************************************/
	if( write_lock.isOwner(NewpkiThread::CurrentThreadId()) ) {
		/**	We own the write lock. **/

		/** We already did a write lock so we're sure to go thru **/
		if( write_ctr > 0 ) {
			write_lock.Wait();	//Take ownership
			write_ctr++;

			return;
		}
		
		/** If we did a read acces we need to make sure no other thread is using
			this read access
		**/
		
		while( read_ctr > 1 ) {
			NewpkiThread::Sleep(50);
		}
	}
	
	write_lock.Wait();	//Take ownership
	write_ctr++;
}

void ReadersWriter::Unlock() const
{
	/***************************************
	"Modified the 24/08/2003 by Sandson (sandson@ifrance.com)
	***************************************/
	
	//Am I a writer
	if (write_ctr > 0)
	{
		((ReadersWriter*)this)->write_ctr--;
		//Release ownership
		write_lock.Post();
		LockerAcc.Unlock();
	}
	else 
	{
		assert(read_ctr > 0);
		if(read_ctr > 0)
		{
			((ReadersWriter*)this)->read_ctr--;
			// Am I the last reader ?
			if (read_ctr == 0)
				write_lock.Post(); //Release ownership
		}

	}
}
