/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

// Entity_KEYSTORE.h: interface for the Entity_KEYSTORE class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ENTITY_KEYSTORE_H
#define ENTITY_KEYSTORE_H


#include "Entity.h"
#include "Entity_KEYSTORE_ASN1.h"


/*!
	This class represents a Key Store Entity
*/
class Entity_KEYSTORE : public Entity  
{
public:
	Entity_KEYSTORE(ENTITY_CONSTRUCTOR_PARAMETERS);
	virtual ~Entity_KEYSTORE();
	bool Load();
	bool Create(const ENTITY_CREATION_DATAS * Params, ADMIN_RESPONSE_BODY * response);
	bool Upgrade(const char * Version);
	bool Init(const ENTITY_SIGNATURE_RESP * init_datas);
	bool ParseAdminCommand(ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest);
	void SpawnStop();

	bool GetWaitingResponses(STACK_OF(STORED_NEWPKI_RESPONSE) * Responses);
	bool GetWaitingRequests(STACK_OF(CRYPTED_NEWPKI_REQUEST) * Requests);
	bool GetProcessedTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool GetWaitingTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool OnRequestSent(unsigned long RequestId, const ASN1_OCTET_STRING * transactionId, const CRYPTED_NEWPKI_REQUEST * req);
	bool OnResponseSent(const STORED_NEWPKI_RESPONSE * resp);
	bool OnNewResponses(const STORED_NEWPKI_RESPONSES * resps);
	bool OnNewRequests(const STORED_NEWPKI_REQUESTS * reqs);
	bool OnNewConf(const ENTITY_CONF_CRYPTED * newConf);

	/*! \brief This function returns the list of types of logs the entity generates.
	 *  \param cLogsType [OUT] The list.
	 */
	static void LogsTypeGet(vector<LOG_MESSAGE_TYPE> & cLogsType);

	/*! \brief This function returns the list of types of ACL the entity handles.
	 *  \param acl_list [OUT] The list.
	 */
	static void GetACL_List(STACK_OF(ASN1_INTEGER) *acl_list);
private:
	bool LoginUser(UserHandle & hUser, int & UserType);
	void LogoutUser(const UserHandle & hUser);
	
	static bool Private_ParseAdminCommand(bool ExecuteCmd, Entity * me_this, vector<LOG_MESSAGE_TYPE> &  mLogsType, ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest, const UserHandle & hUser);

	bool ParseNewConf(const ENTITY_CONF_CRYPTED * newConf, const INTERNAL_PKI_CA * cas);
	bool PersonnalConfFromPEM(const mString & conf_pem);
	bool PersonnalConfToPem(mString & conf_pem);
	
	LOCAL_KEYSTORE_CONF * c_myConf;
	ENTITY_CONF * myConf;
	bool fullInit;

#define MY_KEYSTORE_CONF_PTR body->d.conf
	DECLARE_COMMAND_PARSER(Entity_KEYSTORE);
};

#endif
