/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/



// Entity_PKI.h: interface for the Entity_PKI class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ENTITY_PKI_H
#define ENTITY_PKI_H


#include "Entity.h"
#include "Entity_PKI_ASN1.h"
#include "CA_Handler.h"


#include <mString.h>

#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef enum en_X509_ACL_INIT
{
	X509_ACL_INIT_ERROR		=	-1,
	X509_ACL_INIT_SAME		=	0,
	X509_ACL_INIT_MODIFIED	=	1
}X509_ACL_INIT;

typedef enum en_ENTITY_CONF_UPDATE
{
	ENTITY_CONF_UPDATE_ALL		=	1,
	ENTITY_CONF_UPDATE_ACL		=	2,
	ENTITY_CONF_UPDATE_CRL		=	4,
	ENTITY_CONF_UPDATE_LINKS	=	8,
	ENTITY_CONF_UPDATE_GROUPS	=	16,
	ENTITY_CONF_UPDATE_ADMINS	=	32
}ENTITY_CONF_UPDATE;
#endif


/*!
	This class represents a PKI Entity
*/
class Entity_PKI : public Entity
{
public:
	Entity_PKI(ENTITY_CONSTRUCTOR_PARAMETERS);
	virtual ~Entity_PKI();

	bool GetWaitingResponses(STACK_OF(STORED_NEWPKI_RESPONSE) * Responses);
	bool GetWaitingRequests(STACK_OF(CRYPTED_NEWPKI_REQUEST) * Requests);
	bool GetProcessedTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool GetWaitingTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool OnRequestSent(unsigned long RequestId, const ASN1_OCTET_STRING * transactionId, const CRYPTED_NEWPKI_REQUEST * req);
	bool OnResponseSent(const STORED_NEWPKI_RESPONSE * resp);
	bool OnNewResponses(const STORED_NEWPKI_RESPONSES * resps);
	bool OnNewRequests(const STORED_NEWPKI_REQUESTS * reqs);
	bool OnNewConf(const ENTITY_CONF_CRYPTED * newConf);

	bool Upgrade(const char * Version);
	bool Create(const ENTITY_CREATION_DATAS * Params, ADMIN_RESPONSE_BODY * response);
	bool Init(const ENTITY_SIGNATURE_RESP * init_datas);
	bool Load();
	bool ParseAdminCommand(ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest);
	void SpawnStop();

	/*! \brief This function returns the list of types of ACL the entity handles.
	 *  \param acl_list [OUT] The list.
	 */
	static void GetACL_List(STACK_OF(ASN1_INTEGER) *acl_list);

private:
	bool CheckCaConf(ENTITY_CONF_BODY_CA *Conf);
	bool RenameGroup(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetGroups(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool DelGroup(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool AddGroup(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetGroups(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetEntitiesLinks(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetEntitiesLinks(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetRepositoriesList(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetRepositoriesList(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetConfiguration(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetOfflineState(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetOfflineState(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool RevokeEntityCert(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool RevokeUserCert(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool CreateUser(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool EnumUsers(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool EnumEntities(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SignEntity(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool UserUnlockResourse(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool UserLockResourse(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetEntityMailConf(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetEntityMailConf(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetEntityAcl(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetEntityAcl(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetEntityConf(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetEntityConf(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool SetEntityAudits(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetEntityAudits(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetEntityLogsType(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);

	
	ENTITY_CONF_CRYPTED * EXPORTED_PKI_CONF_get_conf(X509 * cert);
	CA_Handler * GetInternalCA(INTERNAL_CA_TYPE Type);

	bool Add_ENTITY_CONF(char * EntityName, X509 *EntityCert, int Type);
	bool Format_CSR(const char * strCn, const char * strEmail, const X509_PUBKEY * pubKey, PKI_CSR & csrK, const X509 * parent_cert);
	ENTITY_CONF * GetEntityConfiguration(X509 * EntityCert);

	X509_ACL_INIT X509_ACL_init(X509_ACL * acls, int EntityType);
	
	
	bool DeclarePkiAdminCert(const PKI_CERT & cert, const char * Email);
	bool ParseNewConf(const ENTITY_CONF_CRYPTED * newConf, const INTERNAL_PKI_CA * cas);
	bool UpdateEntityConfs(int What, ENTITY_CONF * OneConf = NULL, bool ForceCryptGen=false);
	bool SynchronizeLinks(ENTITY_CONF ** currConf);
	bool STORED_ENTITY_CONF_to_name_x509(const STORED_ENTITY_CONF * conf, ASN1_UTF8STRING ** name, X509 ** cert);
	bool Private_RevokeEntityCert(unsigned long serial);
	bool CheckRaConf(ENTITY_CONF_BODY_RA *Conf);
	bool CheckRepositoryConf(ENTITY_CONF_BODY_REP * Conf);
	
	void Free_PkiConf();
	bool PersonnalConfFromPEM(const mString & conf_pem);
	bool PersonnalConfToPem(mString & conf_pem);

	bool LoginUser(UserHandle & hUser, int & UserType);
	void LogoutUser(const UserHandle & hUser);

	PKI_CONF * pki_conf;

	CA_Handler * RootCa;
	CA_Handler * EntitiesCa;
	CA_Handler * UsersCa;
	CA_Handler * OcspCa;
	PKI_CERT EntityCert;
	PKI_RSA EntityKey;

	ENTITY_ENTRY_INFO * get_LinkToEntity(const ENTITY_LINKS * EntityLink, const char * Name);
	ENTITY_LINKS * get_EntityLink(const ENTITIES_LINKS * Links, const char * Name);

	bool InternalCA_Create(INTERNAL_CA_TYPE Type, const GEN_PRIVATE_KEY * KeyGen, const X509_NAME * Dn, const char * Cn, int days);
	bool InternalCA_Sign(INTERNAL_CA_TYPE Type, const PKI_CSR & csr, PKI_CERT & ResultCert, int Days, bool check_sig);
	bool InternalCA_SignForceDn(INTERNAL_CA_TYPE Type, const X509_PUBKEY * pubKey, const char *cn, const char * email, PKI_CERT & ResultCert);
	bool InternalCA_Revoke(INTERNAL_CA_TYPE Type, unsigned long serial);

#define MY_CONF_PTR pki_conf->my_conf->body->d.pki_conf->d.PKI_CONF_PTR

	DECLARE_COMMAND_PARSER(Entity_PKI);
};

#endif
