/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/

// Entity_RA.h: interface for the Entity_RA class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ENTITY_RA_H
#define ENTITY_RA_H


#include "Entity.h"
#include "Entity_RA_ASN1.h"
#include "RaStore.h"
#include "ClientLDAP.h"
#include "AutoSynchLDAP.h"


/*!
	This class represents a RA Entity
*/
class Entity_RA : public Entity
{
public:
	Entity_RA(ENTITY_CONSTRUCTOR_PARAMETERS);
	virtual ~Entity_RA();

	bool GetWaitingResponses(STACK_OF(CRYPTED_NEWPKI_RESPONSE) * Responses);
	bool GetWaitingRequests(STACK_OF(CRYPTED_NEWPKI_REQUEST) * Requests);
	bool GetProcessedTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool GetWaitingTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool OnRequestSent(unsigned long RequestId, const ASN1_OCTET_STRING * transactionId, const CRYPTED_NEWPKI_REQUEST * req);
	bool OnResponseSent(const STORED_NEWPKI_RESPONSE * resp);
	bool OnNewResponses(const STORED_NEWPKI_RESPONSES * resps);
	bool OnNewRequests(const STORED_NEWPKI_REQUESTS * reqs);
	bool OnNewConf(const ENTITY_CONF_CRYPTED * newConf);

	bool Load();
	bool Create(const ENTITY_CREATION_DATAS * Params, ADMIN_RESPONSE_BODY * response);
	bool Upgrade(const char * Version);
	bool Init(const ENTITY_SIGNATURE_RESP * init_datas);
	bool ParseAdminCommand(ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest);
	void SpawnStop();

	/*! \brief This function returns the list of types of logs the entity generates.
	 *  \param cLogsType [OUT] The list.
	 */
	static void LogsTypeGet(vector<LOG_MESSAGE_TYPE> & cLogsType);

	/*! \brief This function returns the list of types of ACL the entity handles.
	 *  \param acl_list [OUT] The list.
	 */
	static void GetACL_List(STACK_OF(ASN1_INTEGER) *acl_list);
private:
	bool SearchLdap(ADMIN_RESPONSE_BODY *response, int command, const PKI_CERT &UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle &hUser);
	bool DeletePkcs12(ADMIN_RESPONSE_BODY *response, int command, const PKI_CERT &UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle &hUser);
	bool RevokeCert(ADMIN_RESPONSE_BODY *response, int command, const PKI_CERT &UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle &hUser);
	bool RequestCert(ADMIN_RESPONSE_BODY *response, int command, const PKI_CERT &UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle &hUser);
	bool ImportProfile(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool LoginUser(UserHandle & hUser, int & UserType);
	void LogoutUser(const UserHandle & hUser);
	
	bool SendRequest(unsigned long CertReqId, NEWPKI_REQUEST * Request, X509_PUBKEY * recipient);
	
	static bool Private_ParseAdminCommand(bool ExecuteCmd, Entity * me_this, vector<LOG_MESSAGE_TYPE> &  mLogsType, ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest, const UserHandle & hUser);

	bool GetLocalConf(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetUsersGroups(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool EnumProfiles(ADMIN_RESPONSE_BODY *response, int command, const PKI_CERT &UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle &hUser);

	bool ParseNewConf(const ENTITY_CONF_CRYPTED * newConf, const INTERNAL_PKI_CA * cas);
	bool PersonnalConfFromPEM(const mString & conf_pem);
	bool PersonnalConfToPem(mString & conf_pem);
	
	LOCAL_RA_CONF * c_myConf;
	ENTITY_CONF * myConf;

	HashTable_String m_Policies;
	RaStore RaProfilesStore;
	ClientLDAP LdapClient;
	AutoSynchLDAP LdapSynch;

	#define MY_RA_CONF_PTR body->d.conf
	
	DECLARE_COMMAND_PARSER(Entity_RA);
};

#endif
