/*
	Copyright (C) 2003 Frdric Giudicelli (contact_nos@yahoo.com). 
	All rights reserved.

	This product includes cryptographic software written by Eric Young
	(eay@cryptsoft.com)

	This program is released under the GPL with the additional exemption that
	compiling, linking, and/or using OpenSSL is allowed.

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along with
	this program; if not, write to the Free Software Foundation, Inc., 59 Temple
	Place, Suite 330, Boston, MA 02111-1307 USA
*/



// Entity_REPOSITORY.h: interface for the Entity_REPOSITORY class.
//
//////////////////////////////////////////////////////////////////////

#ifndef ENTITY_REPOSITORY_H
#define ENTITY_REPOSITORY_H


#include "Entity.h"
#include "Entity_REPOSITORY_ASN1.h"
#include "RepStore.h"

/*!
	This class represents a Repository Entity
*/
class Entity_REPOSITORY : public Entity
{
public:
	Entity_REPOSITORY(ENTITY_CONSTRUCTOR_PARAMETERS);
	virtual ~Entity_REPOSITORY();

	bool GetWaitingResponses(STACK_OF(CRYPTED_NEWPKI_RESPONSE) * Responses);
	bool GetWaitingRequests(STACK_OF(CRYPTED_NEWPKI_REQUEST) * Requests);
	bool GetProcessedTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool GetWaitingTransactionIds(TRANSACTION_IDS * transactionIDs);
	bool OnRequestSent(unsigned long RequestId, const ASN1_OCTET_STRING * transactionId, const CRYPTED_NEWPKI_REQUEST * req);
	bool OnResponseSent(const STORED_NEWPKI_RESPONSE * resp);
	bool OnNewResponses(const STORED_NEWPKI_RESPONSES * resps);
	bool OnNewRequests(const STORED_NEWPKI_REQUESTS * reqs);
	bool OnNewConf(const ENTITY_CONF_CRYPTED * newConf);

	bool Load();
	bool Create(const ENTITY_CREATION_DATAS * Params, ADMIN_RESPONSE_BODY * response);
	bool ParseAdminCommand(ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest);
	void SpawnStop();
	bool Upgrade(const char * Version);
	bool Init(const ENTITY_SIGNATURE_RESP * init_datas);

	/*! \brief This function returns the list of types of logs the entity generates.
	 *  \param cLogsType [OUT] The list.
	 */
	static void LogsTypeGet(vector<LOG_MESSAGE_TYPE> & cLogsType);

	/*! \brief This function returns the list of types of ACL the entity handles.
	 *  \param acl_list [OUT] The list.
	 */
	static void GetACL_List(STACK_OF(ASN1_INTEGER) *acl_list);
private:
	bool OnNewPkiConf(const EXPORTED_PKI_CONF* conf);
	bool RepositoryKnowsObject(const WAITING_NEWPKI_OBJECT* object, const X509 * Cert);
	bool OnNewObject(WAITING_NEWPKI_OBJECT * object);
	WAITING_NEWPKI_OBJECT * CreateWaitingObject(STORED_NEWPKI_REQUEST *request, STORED_NEWPKI_RESPONSE *response);
	bool CreateNewObject(STORED_NEWPKI_REQUEST *request, STORED_NEWPKI_RESPONSE *response);
	bool SynchronizeWithRepository(PkiClient * ClientPki, const REP_ENTRY_INFO * Repository);
	bool Conf_Flush();
	static bool Private_ParseAdminCommand(bool ExecuteCmd, Entity * me_this, vector<LOG_MESSAGE_TYPE> &  mLogsType, ADMIN_RESPONSE_BODY * response, const PKI_CERT & ClientCert, const ADMIN_REQUEST * AdminRequest, const UserHandle & hUser);

	bool SynchronizeObjects(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool PushConfiguration(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool PushWaitingObject(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetMyConf(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetMyRequests(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetMyResponses(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool ImportRequest(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool ImportResponse(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	bool GetConfiguration(ADMIN_RESPONSE_BODY * response, int command, const PKI_CERT & UserCert, const ADMIN_REQUEST_BODY *body, const UserHandle & hUser);
	
	bool ParseNewConf(const ENTITY_CONF_CRYPTED * newConf, const INTERNAL_PKI_CA * cas);
	bool PersonnalConfFromPEM(const mString & conf_pem);
	bool PersonnalConfToPem(mString & conf_pem);

	static void ThreadCaObjectsPush(const NewpkiThread * Thread, void *param);
	
	bool LoginUser(UserHandle & hUser, int & UserType);
	void LogoutUser(const UserHandle & hUser);

	RepStore RepositoryStore;
	LOCAL_REPOSITORY_CONF * c_myConf;
	ENTITY_CONF * myConf;
	EXPORTED_PKI_CONF * pki_conf;
	NewpkiThread hThreadCaObjectsPush;
	
	ReadersWriter WaitingObjectsLock;
	STACK_OF(WAITING_NEWPKI_OBJECT) * WaitingObjects;

	DECLARE_COMMAND_PARSER(Entity_REPOSITORY);
};

#endif
