  // OcspServers.cpp: implementation of the OcspServers class.
//
//////////////////////////////////////////////////////////////////////

#include "OcspServers.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

OcspServers::OcspServers()
{

}

OcspServers::~OcspServers()
{
	Stop();
	ServersLock.LockWrite();
	std::map< mString, SockServerOCSP* >::iterator i;
	for(i = m_ListServers.begin(); i != m_ListServers.end(); i++)
	{
		// We must unload the server !
		delete i->second;
	}
	m_ListServers.clear();
	ServersLock.Unlock();
}

void OcspServers::Load(PKIX_Central *Central)
{
	m_Central = Central;
}

void OcspServers::Stop()
{
	ServersLock.LockWrite();
	std::map< mString, SockServerOCSP* >::iterator i;
	for(i = m_ListServers.begin(); i != m_ListServers.end(); i++)
	{
		// We must unload the server !
		i->second->Stop();
	}
	ServersLock.Unlock();
}

bool OcspServers::StartServer(const mString &EntityName, const ServerConf &Conf)
{
	ServersLock.LockWrite();
	// Is the server already loaded ?
	if(m_ListServers.find(EntityName) != m_ListServers.end())
	{
		// Was it loaded with same info ?
		if(m_ListServers[EntityName]->GetConf() == Conf)
		{
			// We do nothing
			ServersLock.Unlock();
			return true;
		}
		else
		{
			// We must unload the server !
			m_ListServers[EntityName]->Stop();
			delete m_ListServers[EntityName];
			m_ListServers.erase(EntityName);
		}
	}
	ServersLock.Unlock();


	SockServerOCSP * newServer;
	newServer = new SockServerOCSP();
	if(!newServer)
	{
		NEWPKIerr(PKI_ERROR_TXT, ERROR_MALLOC);
		return false;
	}

	if(!newServer->Load(m_Central, EntityName))
	{
		NEWPKIerr(PKI_ERROR_TXT, ERROR_ABORT);
		delete newServer;
		return false;
	}

	if(!newServer->Start(Conf))
	{
		NEWPKIerr(PKI_ERROR_TXT, ERROR_ABORT);
		delete newServer;
		return false;
	}

	while(!newServer->Started() && !newServer->Failed())
		NewpkiThread::Sleep(100);
	if(newServer->Failed())
	{
		NEWPKIerr(PKI_ERROR_TXT, ERROR_ABORT);
		delete newServer;
		return false;
	}

	ServersLock.LockWrite();
	m_ListServers[EntityName] = newServer;
	ServersLock.Unlock();

	return true;
}

void OcspServers::StopServer(const mString &EntityName)
{
	ServersLock.LockWrite();
	// Is the server already loaded ?
	if(m_ListServers.find(EntityName) != m_ListServers.end())
	{
		// We must unload the server !
		m_ListServers[EntityName]->Stop();
		delete m_ListServers[EntityName];
		m_ListServers.erase(EntityName);
	}
	ServersLock.Unlock();
}

void OcspServers::PrintStats(bool CurrConnectionsStats)
{
	ServersLock.LockRead();
	std::map< mString, SockServerOCSP* >::iterator i;
	for(i = m_ListServers.begin(); i != m_ListServers.end(); i++)
	{
		// We must unload the server !
		i->second->PrintStats(CurrConnectionsStats);
	}
	ServersLock.Unlock();
}

bool OcspServers::IsRunning(const mString &EntityName)
{
	ServersLock.LockRead();
	// Is the server already loaded ?
	if(m_ListServers.find(EntityName) == m_ListServers.end())
	{
		ServersLock.Unlock();
		return false;
	}
	ServersLock.Unlock();
	return true;
}
